
/**
 * Drupal initializer.
 * Launch as behavior and pull variables from config.
 */

// todo: change the Editoria11y event dispatch to run later, with summarized results including post-dismissal

// Prevent multiple inits in modules that re-trigger the document context.
var ed11yOnce;

Drupal.behaviors.editoria11y = {
    attach: function (context, settings) {

        if (context === document && !ed11yOnce && CSS.supports("selector(:is(body))")) {
            // "Supports" selector drops old browsers.
            ed11yOnce = true;

            let ed11yAlertMode = drupalSettings.editoria11y.assertiveness ? drupalSettings.editoria11y.assertiveness : 'polite';
            // If assertiveness is "smart" we set it to assertive if the doc was recently changed.
            const now = new Date();
            if (ed11yAlertMode === "smart" && (now / 1000) - drupalSettings.editoria11y.changed < 60) {
                ed11yAlertMode = "assertive";
            }

            let options = {
              // todo
              // videoContent: 'youtube.com, vimeo.com, yuja.com, panopto.com',
              // audioContent: 'soundcloud.com, simplecast.com, podbean.com, buzzsprout.com, blubrry.com, transistor.fm, fusebox.fm, libsyn.com',
              // dataVizContent: 'datastudio.google.com, tableau',
              // twitterContent: 'twitter-timeline',
              checkRoots: drupalSettings.editoria11y.content_root ? drupalSettings.editoria11y.content_root : false,
              shadowComponents: drupalSettings.editoria11y.shadow_components ? drupalSettings.editoria11y.shadow_components : false,
              ignoreElements: !!drupalSettings.editoria11y.ignore_elements ? `${drupalSettings.editoria11y.ignore_elements}` : false,
              ignoreByKey: {
                  // 'p': false,
                  'h': '.filter-guidelines-item *, #toolbar-administration *, nav *, .block-local-tasks-block *',
                  'img': '[aria-hidden], [aria-label], [aria-labelledby], [aria-hidden] img, [aria-label] img, [aria-labelledby] img', // disable alt text tests on unspoken images
                  'a': '[aria-hidden], #toolbar-administration a, .filter-help > a, .contextual-region > nav a', // disable link text check on disabled links
                  // 'li': false,
                  // 'blockquote': false,
                  // 'iframe': false,
                  // 'audio': false,
                  // 'video': false,
                  // 'table': false,
              },
              alertMode: ed11yAlertMode,
              currentPage: drupalSettings.editoria11y.page_path,
              allowHide: !!drupalSettings.editoria11y.allow_hide,
              allowOK: !!drupalSettings.editoria11y.allow_ok,
              syncedDismissals: drupalSettings.editoria11y.dismissals,
              ignoreAllIfAbsent : !!drupalSettings.editoria11y.ignore_all_if_absent ? drupalSettings.editoria11y.ignore_all_if_absent : false,
              // todo: ignoreAllIfPresent
              preventCheckingIfPresent: !!drupalSettings.editoria11y.no_load ? drupalSettings.editoria11y.no_load : false,
              // todo: preventCheckingIfAbsent
              linkIgnoreStrings: !!drupalSettings.editoria11y.ignore_link_strings ? drupalSettings.editoria11y.ignore_link_strings : `\\(link is external\\)|\\(link sends email\\)`,
              hiddenHandlers: drupalSettings.editoria11y.hidden_handlers,
              theme: !!drupalSettings.editoria11y.theme ? drupalSettings.editoria11y.theme : 'lightTheme',
              embeddedContent: !!drupalSettings.editoria11y.embedded_content_warning ? drupalSettings.editoria11y.embedded_content_warning : false,
              documentLinks: !!drupalSettings.editoria11y.download_links ? drupalSettings.editoria11y.download_links : `a[href$='.pdf'], a[href*='.pdf?'], a[href$='.doc'], a[href$='.docx'], a[href*='.doc?'], a[href*='.docx?'], a[href$='.ppt'], a[href$='.pptx'], a[href*='.ppt?'], a[href*='.pptx?'], a[href^='https://docs.google']`,
            }

            if (editoria11yOptionsOverride) {
                options = editoria11yOptions(options);
            }

            const ed11y = new Ed11y(options);

            let csrfToken = false;
            function getCsrfToken(action, data)
            {
                  {
                        fetch(`${drupalSettings.editoria11y.site_root}session/token`, {
                            method: "GET"
                        })
                  .then(res => res.text())
                  .then(token => {
                        csrfToken = token;
                        postData(action, data).catch(err => console.error(err));
                    })
                  .catch(err => console.error(err));
                }
            }

            let postData = async function (action, data) {
                if (!csrfToken) {
                    getCsrfToken(action, data);
                } else {
                    let url = `${drupalSettings.editoria11y.site_root}editoria11y/api/${action}`;
                    fetch(url, {
                        method: 'POST',
                        headers: {
                            'Content-Type': 'application/json',
                            'X-CSRF-Token': csrfToken,
                        },
                        body: JSON.stringify(data),
                    })
                    .catch((error) => console.error('Error:', error))
                }
            }

            let results = {};
            let oks = {};
            let extractResults = function () {
                Ed11y.results.forEach(result => {
                    if (result[5] !== "ok") {
                        // log all items not marked as OK
                        let testName = result[1];
                        testName = Ed11y.M[testName].title;
                        if (results[testName]) {
                            results[testName] = parseInt(results[testName]) + 1;
                        } else {
                            results[testName] = 1;
                        }
                    }
                    if (result[5] === "ok" || result[5]==="hide") {
                        if (!results[result[1]]) {
                            oks[result[1]] = true;
                        }
                    }
                })
            }

            let sendResults = function () {
                window.setTimeout(function () {
                    extractResults();
                    let data = {
                        page_title: drupalSettings.editoria11y.page_title,
                        page_path: drupalSettings.editoria11y.page_path,
                        page_count: Ed11y.totalCount,
                        entity_type: drupalSettings.editoria11y.entity_type, // node or false
                        route_name: drupalSettings.editoria11y.route_name, // e.g., entity.node.canonical or view.frontpage.page_1
                        results: results,
                        oks: oks,
                    };
                    postData('results/report', data);
                  // Short timeout to let execution queue clear.
                }, 250)
            }
            if (drupalSettings.editoria11y.dismissals) {
                document.addEventListener('ed11yResults', function () {
                    sendResults()
                });
            }
            let sendDismissal = function (detail) {
                if (!!detail) {
                    let data = {};
                    if (detail.dismissAction === 'reset') {
                        data = {
                            page_path: drupalSettings.editoria11y.page_path,
                            route_name: drupalSettings.editoria11y.route_name,
                            dismissal_status: 'reset', // ok, ignore or reset
                        };
                    } else {
                        data = {
                            page_title: drupalSettings.editoria11y.page_title,
                            page_path: drupalSettings.editoria11y.page_path,
                            entity_type: drupalSettings.editoria11y.entity_type, // node or false
                            route_name: drupalSettings.editoria11y.route_name, // e.g., entity.node.canonical or view.frontpage.page_1
                            result_name: Ed11y.M[detail.dismissTest].title, // which test is sending a result
                            result_key: detail.dismissTest, // which test is sending a result
                            element_id: detail.dismissKey, // some recognizable attribute of the item marked
                            dismissal_status: detail.dismissAction, // ok, ignore or reset
                        };
                    }
                    postData('dismiss/' + detail.dismissAction, data);
                }
            }
            if (drupalSettings.editoria11y.dismissals) {
                document.addEventListener('ed11yDismissalUpdate', function (e) {
                    sendDismissal(e.detail)}, false);
            }
        }
    }
};