const ed11yLang={en:{panelCount0:"No issues detected.",panelCountAllDismissed:"All issues hidden.",panelCount1:"One issue detected.",panelCountMultiple:" issues detected.",panelHelp:`
            <p>Assistive devices that can reformat content for people with disabilities need well structured content to work.</p>
            <p>This tool checks for common issues, such as <a href='https://accessibility.princeton.edu/how/content/alternative-text'>missing alternative text</a>, <a href='https://accessibility.princeton.edu/how/content/headings'>jumbled page outlines</a> and <a href='https://accessibility.princeton.edu/how/content/links'>generic link titles</a>, and reminds content creators to check audio and video for closed captioning.</p>
            <p>Note that it <strong>only</strong> checks editorial content; your site's theme needs its own <a href='https://webaim.org/resources/evalquickref/'>accessibility testing</a>.<p>
            <p>Have a bug or feature request? <a href='https://github.com/itmaybejj/editoria11y/issues' class='ed11y-small'>Contact the Editoria11y maintainers</a>.</p>
        `,jumpedToInvisibleTip:"The element with this issue may be hidden. I will outline its nearest visible container",jumpedToAriaHiddenTip:'The element with this issue has an "aria-hidden" attribute and may be invisible or off screen.',elementDismissalHelpOK:`
    "Mark as checked and OK" hides a manual check on one page, for all site editors. The alert may reappear if the page's content changes, or if another editor restores hidden alerts on this page.
    `,elementDismissalHelpHide:`
    "Hide alert" hides a manual check on a single page, for the current user. The alert may reappear if the page content changes.
    `,elementDismissalHelpAll:"To dismiss an alert site-wide, have an administrator adjust the configuration options to ignore this element.",headingExample:`
        <ul>
            <li>Heading level 1
                <ul>
                    <li>Heading level 2
                        <ul><li>Heading level 3 (a <em>subtopic</em>)</li></ul></li>
                    <li>Heading level 2 (a <em>new</em> topic)</li>
                </ul>
            </li>
        </ul>`,headingLevelSkipped:{title:"Manual check: was a heading level skipped?",tip:(e,t)=>`<p>Headings and subheadings are the page's table of contents. The <em>numbers</em> indicate indents in a nesting relationship:</p>
            ${Ed11y.M.headingExample}
            <p>This heading is level ${t}; the previous heading was level ${e}. 
            A screen reader user will not know if level ${parseInt(t-1)} is missing, or if they were not paying attention and missed it.
            They may to go back and check.</p>
            <p>To fix: adjust levels to form a continuous outline.</p>
            `},headingEmpty:{title:"Heading tag without any text",tip:()=>`<p>Headings and subheadings create a navigable table of contents for assistive devices. "Blank" headings create confusing gaps. The heading's <strong><em>number</em></strong> indicates its <strong><em>depth</em></strong> in the page outline; e.g.:</p>
            ${Ed11y.M.headingExample}
            <p><strong>To fix:</strong> add text to this heading, or delete it.</p>
            `},headingIsLong:{title:"Manual check: long heading",tip:()=>`<p>Headings should be brief and clear. Assistive devices use them to create a navigable table of contents for the page. The heading's <strong><em>number</em></strong> indicates its <strong><em>depth</em></strong> in the page outline; e.g.:</p>  
            ${Ed11y.M.headingExample}
            <p><strong>To fix:</strong> shorten this heading if possible, or remove the heading style if it was only applied to this text to provide visual emphasis.</p>
            `},blockquoteIsShort:{title:"Manual check: is this a blockquote?",tip:()=>"<p>Blockquote formatting tells screen readers that this text should be announced as a quotation. This was flagged because short blockquotes are sometimes actually headings that belong in the page outline. If that is the case here, please use heading formatting instead.</p>"},altAttributeExample:`<p>Note that a good alt conveys what an image communicates, not what it contains. For example, a picture of a child kicking a ball might be there to show a generic scene, or the kick, or the child or the ball:</p>
            <ul>
                <li>A child kicking a ball</li>
                <li>AJ's game-winning kick curved in from the left sideline!</li>
                <li>AJ</li>
                <li>The "medium" ball is the right size for this 9-year-old child</li>
            </ul>`,altAttributeProvided:e=>`<p>This image's alt text is: "<strong>${e}</strong>"</p>`,altMissing:{title:"Image has no alternative text attribute",tip:()=>`<p>When screen readers encounter an image with no alt attribute, they dictate the url of the image file instead, often one letter at a time.
            Please set this image's alternative text to a concise description of what this image means in this context.</p>
            `+Ed11y.M.altAttributeExample},altNull:{title:"Manual check: image is marked as decoration",tip:()=>`<p>This image provides no text alternative; screen readers will skip over the image without speaking anything. This may be correct!</p>
        <p>But if this image is meaningful, provide alt text.</p> 
            `+Ed11y.M.altAttributeExample},altURL:{title:"Image's text alternative is a URL",tip:e=>`<p>Please set this image's alternative text to a concise description of what this image means in this context.</p>
            ${Ed11y.M.altAttributeExample}
            `+Ed11y.M.altAttributeProvided(e)},altURLLinked:{title:"Linked image's text alternative is a URL",tip:e=>`<p>When a link includes an image, the image's alt text becomes part of the link text announced by screen readers.
            Links should clearly and concisely describe their destination; a URL (usually pronounced by the screen reader one letter at a time) does not.</p>
            <ul>
                <li>Good link text: "About us"</li>
                <li>Bad link text: "H T T P S colon forward slash forward slash example dot com forward slash aye bee oh you tee you ess</li>
            </ul>
            `+Ed11y.M.altAttributeProvided(e)},altImageOf:{title:"Manual check: possibly redundant text in alt",tip:e=>`<p>Since screen readers automatically announce they are describing an image, 
            words like "image," "photo" or "graphic" are only helpful 
            in a text alternative if they describe what is contained in the image, rather
            than repeating the fact that the image is an image.</p>
            <ul><li>Redundant: "<em>Photo of</em> my new spoon."</li>
            <li>Helpful: "Cracked and faded <em>photo of</em> the first computer."</li></ul>
            `+Ed11y.M.altAttributeProvided(e)},altImageOfLinked:{title:"Manual check: possibly redundant text in linked image",tip:e=>`<p>Links should clearly and concisely describe their destination. Since words like "image," "graphic" or "photo" are redundant in text alternatives (screen readers already identify the image as an image), they often indicate that the image's text alternative is describing the image instead of the link.</p>
            <ul>
                <li>Good link text: "About us"</li>
                <li>Bad link text: "Stock photo of five people jumping and high fiving around a conference table, image"</li>
            </ul>
            <hr>
            `+Ed11y.M.altAttributeProvided(e)},altDeadspace:{title:"Image's text alternative is unpronouncable",tip:e=>`<p>This image's alt consists of only silent characters (spaces and quotation marks). It will be announced by screen readers as as part of the link's text, but the description of what the image is will be unintelligible. Please set this image's alternative text to something that describes the link's destination, or provide a <em>completely</em> empty alt (alt="") if the image should not be mentioned at all.</p>
            <ul>
                <li>Good link text: "About us"</li>
                <li>Bad link text: "About us, image: [short confusing silence]"</li>
            </ul>
            `+Ed11y.M.altAttributeProvided(e)},altDeadspaceLinked:{title:"Linked Image's text alternative is unpronouncable",tip:e=>`<p>This image's alt consists of only silent characters (spaces and quotation marks). It will be announced by screen readers as as part of the link's text, but the description of what the image is will be unintelligible. Please set this image's alternative text to something that describes the link's destination, or provide a <em>completely</em> empty alt (alt="") if the image should not be mentioned at all.</p>
            <ul>
                <li>Good link text: "About us"</li>
                <li>Bad link text: "About us, image: [short confusing silence]"</li>
            </ul>
            `+Ed11y.M.altAttributeProvided(e)},altEmptyLinked:{title:"Linked Image has no alt text",tip:()=>`<p>When a link is wrapped around an image, the image's alt text provides the link's title for screen readers</p>
        <p>Please set this image's alternative text to something that describes the link's destination, or add text to the link.</p>
            <ul>
                <li>Good linked alt: "Meaningful link tips"</li>
                <li>Bad linked alt: "Three happy dogs rolling in the grass"</li>
            </ul>
            `},altLong:{title:"Manual check: very long alternative text",tip:e=>`<p>Image text alternatives are announced by screen readers as a single run-on sentence; listeners must listen to the entire alt a second time if they miss something. When more that a few words are needed to describe an image, it is usually better to provide and reference a <em>visible</em> text alternative, an approach that is often preferred by mobile device users and readers with low-vision. For example:</p>
            <ul><li>"Event poster; details provided in caption"</li>
            <li>"Chart showing our issues on this page going to zero; details in table"</li></ul>
            ${Ed11y.M.altAttributeProvided(e)}
            `},altLongLinked:{title:"Manual check: very long alternative text in linked image",tip:e=>`<p>The alt text on a linked image is used to describe the link destination. Links should be brief, clear and concise, as screen reader users often listen to the list of links on the page to find content of interest. Long alternative text inside a link often indicates that the image's text alternative is describing the image instead rather than the link.</p>
            ${Ed11y.M.altAttributeProvided(e)}
            `},altPartOfLinkWithText:{title:"Manual check: link contains both text and an image",tip:e=>`<p>When a link includes an image, the image's alt text becomes part of the link.
            This can be confusing if the image's text alternative is adding irrelevant details.</p>
            <ul>
                <li>Confusing: "Link, 'Stock photo of five people jumping and high fiving around a conference table, image', About us</li>
                <li>Ideal: "Link, About us" (image was marked decorative with a blank alt and not announced)</li>
            </ul>
            ${Ed11y.M.altAttributeProvided(e)}
            `},linkNoText:{title:"Link with no accessible text",tip:()=>`<p>This link is either invisible and empty (e.g., a linked space character), or wrapped around something with no text alternative (an image with no alt attribute).</p>
            <p>Screen readers will not be able to read this, speaking either an uninformative silence "Link, [...awkward pause where the link title should be...]," or the URL, spelled out character by character: "Link, H-T-T-P-S forward-slash forward-slash example dot com"</p>
            <p>To fix: add text if this should be a link; delete the spaces if not.</p>`},linkTextIsURL:{title:"Manual check: is this link text a URL?",tip:e=>`<p>Readers often skim by link titles. This is especially true of screen reader users, who navigate using a list of on-page links.</p>
         <p>A linked URL expects the reader to read the preceding paragraph to figure out the link's purpose from context.</p>
            <ul>
                <li>Meaningful and concise link text: "Tips for writing meaningful links"</li>
                <li>Linked URL, as pronounced by a screen reader: "H T T P S colon forward-slash forward-slash example dot com forward-slash tips forward-slash meaningful-links"</li>
            </ul>
                <p>This link's text is:<br> <strong>${e}</strong></p>
                `},linkTextIsGeneric:{title:"Manual check: is this link meaningful and concise?",tip:e=>`<p>Readers often skim by link titles. This is especially true of screen reader users, who navigate using a list of on-page links.</p>
                <p>Generic text like "click here," "read more" or "download" expect the reader to figure out the link's purpose from context.</p>
                <ul><li>Not meaningful: "To learn more about meaningful links, <a href="https://www.google.com/search?q=writing+meaningful+links">click here</a>.</li>
                <li>Not concise: "<a href="https://www.google.com/search?q=writing+meaningful+links">Click here to learn more about meaningful links</a>"</li>
                <li>Ideal: "Learn about <a href="https://www.google.com/search?q=writing+meaningful+links">meaningful links"</a></strong></li></ul>
                <p>This link's text is: <strong>${e}</strong></p>`},linkDocument:{title:"Manual check: is the linked document accessible?",tip:()=>`<p>This automated checker helps ensure Web pages contain the features assistive technologies need to make content accessible for everybody; things like headings, table headers and image alternative text. It is not able to check linked documents.</p>
                <p>If the linked document cannot be converted to a Web page (something usually preferred by mobile device and screen reader users), please make sure it at least contains structural tags and image alt text. See tips for remediating:
                <ul>
                    <li><a href='https://webaim.org/techniques/word/'>text documents</a></li>
                    <li><a href='https://webaim.org/techniques/powerpoint/'>slideshows</a></li>
                    <li><a href='https://webaim.org/techniques/acrobat/'>documents intended for printers instead of screens (PDF)</a></li>
                </ul>`},linkNewWindow:{title:"Manual check: is opening a new window expected?",tip:()=>`<p>Normally readers can choose to open a link in the same window or a new window. When a link forces the browser to open in a new window without warning, it can be confusing, especially for assistive device users who may not realize a new window has opened and wonder why their browse "back" button is suddenly disabled, or who need many clicks to get back to the previous window.</p>
                <p>There are generally only two situations where it is not confusing or annoying to force a link to open a new window:</p>
                <ul>
                    <li>When the user is in a multi-step form (e.g., a store checkout process), and they would lose their place if they followed a link.</li>
                    <li>When the user is warned by text or an icon with a text-equivalent for screen readers the link will open in a new window.</li>
                </ul>
                `},tableNoHeaderCells:{title:"Table has no header cells",tip:()=>`
                <p>Tables are announced by screen readers as navigable data sets. Screen readers repeat row and column headers as needed to orient users while reading content cells.</p>
                <p>To fix:</p>
                <ul><li>If this table contains information that is meaningfully organized by row and column, edit the table's properties and specify whether the headers are in the first row, column or both.</li>
                <li>If this table does not contain rows and columns of data, but is simply being used for visual layout, it would be best to remove it. Tables overflow the page rather than reflowing the text to fit on mobile devices, and should only be used when horizontal relationships are necessary to understand the content.</li></ul>
            `},tableContainsContentHeading:{title:"Content heading inside a table",tip:()=>`<p>Content headings ("Heading 1", "Heading 2") form a navigable table of contents for screen reader users,  
        labelling all content <strong>until the next heading</strong>. Table headers label specific columns or rows within a table.</p> 
            <p></p>
            <table><tr><th>1</th><th>2</th><th>3</th><td rowspan="2">To illustrate: a <strong>table</strong> header in cell 2 would only label its column: cell B. <br><br>
            A <strong>content</strong> heading in cell 2 would label all subsequent text, reading from left to right: cells 3, A, B and C, as well as this text!</td></tr>
            <tr><td>A</td><td>B</td><td>C</td></table>
            <p>To fix: remove heading formatting on text inside table cells.</p>
            `},tableEmptyHeaderCell:{title:"Empty table header cell",tip:()=>`
                <p>When exploring tables, screen readers repeat table header cells as needed to orient users. 
                Without headers, it is very easy to get lost; screen reader users have to count columns and rows and try to remember which columns went with which rows.</p>
                <p>To fix: make sure each header cell in this table contains text.</p>
            `},textPossibleList:{title:"Manual check: should this have list formatting?",tip:e=>`<p>List formatting is structural:</p> 
            <ol><li>List formatting indents and reflows on overflow. Text aligns vertically with text, rather than the "${e}"</li>
            <li>Lists are machine-readable. Screen readers can orient their users, announcing this as "list item, 2 of 2."</li></ol>
            <p>3. Whereas this unformatted item (just a number, typed as text) is not visually or audibly included in the list.</p>
            <p>To fix: if this "${e}" starts a list, replace it with list formatting.</p>
            `},textPossibleHeading:{title:"Manual check: should this be a heading?",tip:()=>`<p>Headings and subheadings create a navigable table of contents for assistive devices. The heading's <strong><em>number</em></strong> indicates its <strong><em>depth</em></strong> in the page outline; e.g.:</p>
            ${Ed11y.M.headingExample}
            <p>If this all-bold text is functioning visually as a heading, please replace the bold formatting with the appropriately numbered heading.</p>
            `},textUppercase:{title:"Manual check: is this uppercase text needed?",tip:()=>`<p>UPPERCASE TEXT CAN BE MORE DIFFICULT TO READ FOR MANY PEOPLE, AND IS OFTEN INTERPRETED AS SHOUTING.</p>
         <p>Consider using sentence case instead, and using bold text or font changes for visual emphasis, or structural formatting like headings for emphasis that will also be announced by screen readers.</p>`},embedVideo:{title:"Manual check: is this video accurately captioned?",tip:()=>`<p>If a recorded video contains speech or meaningful sounds, it must be captioned.</p>
            <p>Note that machine-generated captions are an excellent start, but must be proofread and edited to identify who is speaking before being considered an equal alternative.</p>`},embedAudio:{title:"Manual check: is an accurate transcript provided?",tip:()=>`<p>If this audio contains speech, a text alternative must be provided, either as content on this page, a link or a download.</p>
            <p>Note that machine-generated transcripts are an excellent start, but must be proofread and edited to identify who is speaking before being considered an equal alternative</p>`},embedVisualization:{title:"Manual check: is this visualization accessible?",tip:()=>`<p>Visualization widgets are often difficult or impossible for assistive devices to operate, and can be difficult to understand for readers with low vision or colorblindness.</p>
            <p>Unless this particular widget has an adjustable font size, high visual contrast, can be operated by a keyboard and can be understood by a screen reader, assume that an alternate format (text description, data table or downloadable spreadsheet) should also be provided.</p>`},embedTwitter:{title:"Manual check: is this embed a keyboard trap?",tip:()=>`<p>If embedded feeds are set to show a high number of items, keyboard users may have to click the tab key dozens or hundreds of times to exit the component.</p>
            <p>Check to make sure only a small number of items auto-load immediately or while scrolling. Having additional items load on request ("show more") is fine.</p>`},embedCustom:{title:"Manual check: is this embedded content accessible?",tip:()=>"<p>Please make sure images inside this embed have alt text, videos have captions, and interactive components can be <a href='https://webaim.org/techniques/keyboard/'>operated by a keyboard</a>.</p>"}}};class Ed11yElementAlt extends HTMLElement{constructor(){super()}connectedCallback(){if(!this.initialized){const n=this.attachShadow({mode:"open"});let e=document.createElement("div");e.setAttribute("class","wrapper");var i=Ed11y.imageAlts[this.dataset.ed11yImg];let t=document.createElement("span");t.textContent=i[2],t.classList.add(i[3]),e.appendChild(t);const s=document.createElement("style");s.textContent=Ed11y.baseCSS+`
        :host {
          position: absolute;
        }
        div {
          position: absolute;
          bottom: 0;
          left:0;
          right:0;
        }
        span {
          background: ${Ed11y.color.primary}f3;
          color: ${Ed11y.color.primaryText};
          z-index: 100;
          font-weight: 500;
          padding: 6px;
          position: absolute;
          bottom: 12px;
          left: 0;
          right: 0;
          box-shadow: 0 1px, 0 -1px; 
        }
        .warning { background: ${Ed11y.color.warning}; color: #111;}
        .error { background: ${Ed11y.color.bgHighlight};}
      `,n.appendChild(s),n.appendChild(e),this.initialized=!0}}}customElements.define("ed11y-element-alt",Ed11yElementAlt);class Ed11yElementPanel extends HTMLElement{constructor(){super()}template(){return`
    <div class='buttonbar' role='tablist' aria-label='Editorial Ally panel controls'>
      <button role="tab" id='issues' aria-selected='true'>
          Issues
      </button>
      <button role="tab" id='headings' aria-selected='false'>
          Outline
      </button>
      <button role="tab" id='alts' aria-selected='false'>
          Alt Text
      </button>
      <button role="tab" id='help' aria-selected='false' aria-controls='help-tab' title='About this tool'>?</button>
      <button role="tab"  id='minimize' aria-selected='false' title='Minimize panel' aria-label="minimize" aria-pressed="false"><span>&ndash;</span></button>
      <button type='button' id='toggle'><span class='close'>&times;</span><span class='open'><span class='icon'></span><span class='toggle-count'></span></span></button>
    </div>
    <div class="content">
      <div id='issues-tab' tabindex="0" role="tabpanel" class="show" aria-labelledby='issues'>
          <div>
              <div class='content-text'><span class='count'>No</span> <span class='content-type'>accessibility errors detected</span>.</div>
              <div aria-live='polite' class='announce hidden'></div>
          </div>
          <div class='jumplinks'>
            <button class='jump prev' data-ed11y-goto='0'><span aria-hidden='true'>« </span><span class='jump-prev'>Previous</span></button>
            <button class='jump next' data-ed11y-goto='0'><span class='jump-next'>First</span> <span aria-hidden='true'> »</span></button>
            <button id='restore' hidden>Show hidden alerts</button>
          </div>
        </div>
      <div id='headings-tab' role="tabpanel" class="hidden" aria-labelledby='headings' tabindex='0'>
        <p>Check that this forms <a href='https://accessibility.princeton.edu/how/content/headings'>a complete outline</a>:</p>
        <ul id='outline'></ul>
      </div>
      <div id='alts-tab' role="tabpanel" class="hidden" aria-labelledby='alts' tabindex='0'>
        <p>Check <a href='https://accessibility.princeton.edu/how/content/alternative-text'>alt text</a>, <a href='https://accessibility.princeton.edu/how/content/images-text'>images of text</a>, &amp; <a href='https://webaim.org/techniques/captions/'>captions</a>.</p>
        <ul id='alt-list'></ul>
      </div>
      <div id='help-tab' role="tabpanel" class="hidden" aria-labelledby='help' tabindex='0'>
      // todo inject content
      </div>
    </div>
    `}connectedCallback(){if(!this.initialized){this.setAttribute("style","outline: 0;");const i=this.attachShadow({mode:"open"});let e=document.createElement("div"),t=(e.setAttribute("class","wrapper"),e.innerHTML=this.template(),document.createElement("style"));t.textContent=Ed11y.baseCSS+`
        :host {
          position: fixed;
          right: 1%;
          bottom: 1%;
          opacity: 0;
          transition: opacity .25s ease-in;
          z-index: 9998;
        }
        :host:focus, div:focus {
          outline: transparent;
        }
        .wrapper {
          width: clamp(160px, 23em, 92vw);
          background: ${Ed11y.color.bg};
          color: ${Ed11y.color.text};
          border-radius: 3px;
          box-shadow: 1px 1px 4px 2px ${Ed11y.color.text}77;
          padding: 2px;
        }
        a {
          color: inherit;
        }
        .content {
          padding: 4px 4px 4px 12px;
          border: 2px solid ${Ed11y.color.primary};
          border-top: 0;
          border-radius: 0 0 3px 3px;
          color: ${Ed11y.color.text};
          max-height: max(240px, 50vh);
          overflow: auto;
        }
        @media (min-width: 400px) {
          #toggle-tab {
            display: grid;
            grid-gap: 1em;
            grid-template-columns: 16em 1fr;
          }
        }
        .hidden {
          display:none;
        }
        .buttonbar {
          display: flex;
        }
        button {
          margin: 0;
          background: ${Ed11y.color.primary};
          color: ${Ed11y.color.primaryText};
          padding: 6px 4px;
          font-family: inherit;
          font-size: 11px;
          font-weight: 500;
          border: 0;
          border-right: 1px solid ${Ed11y.color.primaryText}55;
          text-align: center;
          flex: auto;
          cursor: pointer;
        }
        button:hover {
          background: ${Ed11y.color.text};
        }
        #shut-panel {
          border-right: 0;
        }
        .buttonbar button {
          color: ${Ed11y.color.primaryText}ee;
        }
        .buttonbar button:hover {
          background: ${Ed11y.color.secondary};
        }
        .buttonbar button:first-child {
          border-radius: 3px 0 0 0; 
          border-left: 2px solid transparent;
        }          
        .buttonbar button:last-child {
          border-radius: 0 3px 0 0;
        }
        .buttonbar button[aria-selected="true"] {
          background: ${Ed11y.color.activeTab};
          box-shadow: inset 0 0 0 1px ${Ed11y.color.primary}, inset 0 -2px ${Ed11y.color.primary}22;
          color: ${Ed11y.color.text};
          border: 0;
        }
        .buttonbar button + button[aria-selected="true"] {
          margin-left: -1px;
        }
        .shut {
          border-radius: 100%;
          background: transparent;
          width: auto;
          box-shadow: none;
        }
        .shut button, 
        .shut .content, 
        .active .toggle-count,
        .shut .close {
          display: none;
        }
        .shut #toggle {
          display: block;
          min-width: 36px;
          height: 36px;
          border-radius: 100%;
          padding: 8px;
          border: 0;
        }
        .pass.shut #toggle {
          font-size: 16px;
          background: ${Ed11y.color.primary};
          line-height: 1;
          box-shadow: inset 0 0 0 2px ${Ed11y.color.primary}, inset 0 0 0 4px #fffe;
          font-family: georgia, serif;
        }
        .pass.shut #toggle:hover {
          box-shadow: inset 0 0 0 2px #fffe, 0 0 0 2px ${Ed11y.color.primary}; 
        }
        .shut.warnings #toggle {
          background-color: ${Ed11y.color.warning};
          color: #000b;
          box-shadow: inset 0 0 0 2px ${Ed11y.color.warning}, inset 0 0 0 3px #000b, 0 0 2px #000;
        }
        .shut.warnings #toggle:hover {
          box-shadow: inset 0 0 0 2px ${Ed11y.color.warning}, inset 0 0 0 3px #000b, 0 0 0 3px #000b;
        }
        .shut.errors #toggle {
          color: ${Ed11y.color.alert};
          box-shadow: inset 0 0 0 1px ${Ed11y.color.alert}, inset 0 0 0 2px #fefefe, inset 0 0 0 6px ${Ed11y.color.alert}, 1px 1px 5px 0 rgba(0,0,0,.5);
          background: #fefefe;
        }
        .shut.errors #toggle:hover {
          box-shadow: inset 0 0 0 1px #b80519, inset 0 0 0 2px #fefefe, inset 0 0 0 6px #b80519, 0 0 0 3px ${Ed11y.color.alert}, 0 0 0 4px transparent;
        }
        .shut .toggle-count {
          display: block;
        }
        .jumplinks {
          text-align: right;
          width: min(12em, 50%);
        }
        .jumplinks button {
          min-width: max(7.25em, calc(49% - 3px));
        }
        .content button {
          padding: 5px 11px;
          border-radius: 2px;
          background: inherit;
          color: inherit;
          border: 1px ${Ed11y.color.button}55 solid;
          margin: 2px 0 2px 1px;
        }
        .jump.prev {
          min-width: 81px;
        }
        .jump.next {
          min-width: 60px;
        }
        #restore {
          min-width: min(146px, 100%);
        }
        .content button:hover {
          background: ${Ed11y.color.bg};
          color: ${Ed11y.color.text};
          box-shadow: inset 0 0 0 1px ${Ed11y.color.text};
        }

        .warning {
          background: ${Ed11y.color.warning};
          color: #111;
        }
        .error {
          background: ${Ed11y.color.bgHighlight};
          color: #fff;
        }
        #issues-tab:not(.hidden) {
          display: flex;
          
          gap: 8px;
        }
        #issues-tab > div {
          flex: 1 1 auto;
          align-self: center;
        }
        #outline, #alt-list {
          list-style: none;
          padding: 0;
          margin: 0;
        }
        #outline li {
          padding: 5px;
        }
        #alt-list li {
          display: flex;
          flex-wrap: wrap;
          gap: 1em;
          padding: 8px;
          margin: 8px 0;
          box-shadow: 0 4px ${Ed11y.color.bg}, 0 5px ${Ed11y.color.primary}22;
        }
        #alt-list li span {
          flex: 0 1 calc(100% - 100px);
        }
        #alt-list li img {
          flex: 0 1 80px;
          width: 80px;
          align-self: flex-start;
        }
      `,i.appendChild(t),i.appendChild(e),Ed11y.panel=e,Ed11y.panelToggle=e.querySelector("#toggle"),Ed11y.panelMessage=e.querySelector(".content-text"),Ed11y.panelCount=e.querySelector(".count"),Ed11y.panelJumpNext=e.querySelector(".jump.next"),Ed11y.panelJumpPrev=e.querySelector(".jump.prev"),Ed11y.panelJumpNext.addEventListener("click",this.jumpTo),Ed11y.panelJumpPrev.addEventListener("click",this.jumpTo),Ed11y.restoreDismissed=e.querySelector("#restore"),Ed11y.restoreDismissed.addEventListener("click",function(){Ed11y.clearDismissals()}),Ed11y.announce=e.querySelector(".announce"),Ed11y.panelTabs=e.querySelectorAll(".buttonbar button"),Ed11y.panelTabs.forEach(e=>{e.addEventListener("click",this.handleBarClick)}),this.initialized=!0}}jumpTo(e){e.preventDefault(),Ed11y.resetClass(["ed11y-hidden-highlight"]),Ed11y.elements.jumpList||Ed11y.buildJumpList();e=parseInt(this.dataset.ed11yGoto);let t=Ed11y.elements.jumpList[e];e=Ed11y.results[t.getAttribute("data-ed11y-result")];let i=window.getComputedStyle(document.querySelector("body"));var n=t.getBoundingClientRect().top-parseInt(i.getPropertyValue("padding-top"))-50;let s=!1;e=e[3];let a;"beforebegin"===e?a=Ed11y.nextUntil(t,":not(ed11y-element-result)"):"afterbegin"===e&&(a=t.parentElement);let o;0<Ed11y.options.hiddenHandlers.length&&a.closest(Ed11y.options.hiddenHandlers)?(document.dispatchEvent(new CustomEvent("ed11yShowHidden",{detail:{id:t.getAttribute("id")}})),Ed11y.hiddenHandled=t.getAttribute("id"),window.setTimeout(function(){var e=t.getBoundingClientRect().top-parseInt(i.getPropertyValue("padding-top"))-50;document.querySelector("html, body").animate({scrollTop:e},1),t.setAttribute("data-ed11y-action","open"),window.setTimeout(function(){Ed11y.hiddenHandled=!1},500)},500,t)):(Ed11y.visible(a)?a.closest('[aria-hidden="true"]')&&(s=(s=a.closest('[aria-hidden="true"]')).closest(':not([aria-hidden="true"])'),o=Ed11y.M.jumpedToAriaHiddenTip):(s=Ed11y.firstVisibleParent(a),o=Ed11y.M.jumpedToInvisibleTip),s&&(alert(o),s.classList.add("ed11y-hidden-highlight")),document.querySelector("html, body").animate({scrollTop:n},1),t.shadowRoot.querySelector(".toggle").focus(),t.setAttribute("data-ed11y-action","open"))}handleBarClick(e){e.preventDefault();var t=e.currentTarget.getAttribute("id");switch(t){case"toggle":Ed11y.togglePanel();break;case"minimize":Ed11y.minimize();break;default:Ed11y.switchPanel(t)}}static get observedAttributes(){return["style"]}attributeChangedCallback(e,t,i){if("style"===e&&!0===this.initialized){let e=this.shadowRoot?.querySelector("div");e.setAttribute("style",i)}}}customElements.define("ed11y-element-panel",Ed11yElementPanel);class Ed11yElementHeadingLabel extends HTMLElement{constructor(){super()}connectedCallback(){if(!this.initialized){const s=this.attachShadow({mode:"open"});let e=document.createElement("div");e.setAttribute("class","wrapper");var i=this.dataset.ed11yHeadingOutline,i=Ed11y.headingOutline[i],n=(e.innerHTML="H"+i[1],!!i[2]);e.classList.add("issue"+n);let t=document.createElement("style");n=Math.max(52-8*i[1],12);t.textContent=Ed11y.baseCSS+`
        :host {
          position: absolute;
          margin-top:-.5em;
        }
        .wrapper {          
          background: ${Ed11y.color.primary}ee;
          color: ${Ed11y.color.primaryText};
          box-shadow: 0 0 0 1px ${Ed11y.color.bg}ee, 0 0 0 4px ${Ed11y.color.primary}ee, 1px 1px 5px 2px #000;
          padding: 0 .5em;
          line-height: 1.2;
          border-radius: 2px;
          margin-left: 35px;
          font-weight: 400;
          font-size: ${n}px;
        }
      `,s.appendChild(t),s.appendChild(e),this.initialized=!0}}}customElements.define("ed11y-element-heading-label",Ed11yElementHeadingLabel);class Ed11yElementResult extends HTMLElement{constructor(){super()}connectedCallback(){if(!this.initialized){this.open=!1,this.setAttribute("style","outline: 0px solid transparent;");const t=this.attachShadow({mode:"open"});this.resultID=this.dataset.ed11yResult,this.result=Ed11y.results[this.resultID],this.wrapper=document.createElement("aside"),this.dismissable=!1!==this.result[4],this.wrapper.classList.add("wrapper"),this.wrapper.classList.add("ed11y-result"),this.toggle=document.createElement("button"),this.toggle.setAttribute("class","toggle");var e=this.dismissable?"manual check needed":"alert";this.toggle.setAttribute("aria-label",`Accessibility issue ${this.resultID}, `+e),this.toggle.setAttribute("aria-expanded","false"),this.toggle.setAttribute("aria-haspopup","dialog"),this.toggle.setAttribute("data-ed11y-result",this.dataset.ed11yResult),this.toggle.setAttribute("data-ed11y-ready","false"),this.wrapper.appendChild(this.toggle),this.toggle.addEventListener("click",this.toggleClick),this.addEventListener("mouseover",this.handleHover),this.tipNeedsBuild=!0;const i=document.createElement("style");e=this.toggleImage();i.textContent=Ed11y.baseCSS+e+`
        :host {
          position: absolute;
          opacity: 0;
          transition: opacity .25s ease-in;
          z-index: 9998;
        }
        :host([data-ed11y-open='true']) {
          z-index: 9999;
        }
        .wrapper {
          width: 44px;
          height: 44px;
          overflow: visible;
          color: ${Ed11y.color.text};
        }
        button {
          margin: 0;
          border: 0;
          background: inherit;
          font-family: inherit;
          font-size: 11px;
          font-weight: 600;
          text-align: center;
          cursor: pointer;
          color: ${Ed11y.color.primaryText};
          background: ${Ed11y.color.tipHeader};
        }
        .toggle {
          display: block;
          border: 0;
          border-radius: 50%;
          position: absolute;
          margin: 0 0 0 10px;
          z-index: 1;
          padding: 0;
          vertical-align: middle;
          cursor: pointer;
          width: 32px;
          height: 32px;
          line-height: 1;
          font-size: 16px;
        }
        button:focus-visible {
          outline: 2px solid transparent;
          box-shadow: inset 0 0 0 2px ${Ed11y.color.focusRing}, 0 0 0 3px ${Ed11y.color.primary};
        }
      `,t.appendChild(i),t.appendChild(this.wrapper),this.initialized=!0}}toggleImage(){var e=`
      .toggle {
        box-shadow: inset 0 0 0 2px ${Ed11y.color.warning}, inset 0 0 0 3px #444, inset 0 0 0 6px ${Ed11y.color.warning}, 1px 1px 5px 0 rgba(0,0,0,.5);
        background: ${Ed11y.color.warning};
        color: #333;
      }
      .toggle::before {
        content: "?";
      }
      .toggle:hover, .toggle[aria-expanded='true'] {
        border: 2px solid ${Ed11y.color.primary};
      }`,t=`
      .toggle {
        box-shadow: inset 0 0 0 1px ${Ed11y.color.alert}, inset 0 0 0 2px #fefefe, inset 0 0 0 6px #b80519, 1px 1px 5px 0 rgba(0,0,0,.5);
        background: #fefefe;
        color: ${Ed11y.color.alert};
      }
      .toggle:hover, .toggle[aria-expanded='true'] {
        box-shadow: inset 0 0 0 1px ${Ed11y.color.alert}, inset 0 0 0 2px #fefefe, inset 0 0 0 6px #b80519, 0 0 0 2px ${Ed11y.color.primary}, 0 0 0 3px transparent;
      }
      .toggle::before {
        content: "!";
      }`;return this.dismissable?e:t}toggleClick(e){e.preventDefault();let t=this.getRootNode().host;e="false"===t.getAttribute("data-ed11y-open")?"open":"close";t.setAttribute("data-ed11y-action",e),"open"==e&&window.setTimeout(function(){let e=document.querySelector('ed11y-element-tip[data-ed11y-open="true"]');e.shadowRoot.querySelector(".close").focus()},500)}closeOtherTips(){Ed11y.findElements("openTips",'[data-ed11y-open="true"]'),Ed11y.elements.openTips&&Array.from(Ed11y.elements.openTips).forEach(e=>{e.setAttribute("data-ed11y-action","close")})}tipDOM(e,t,i){return`>
      <div class="title" id="tip-title-${e}">${t}</div>
      <div class="message">${i}</div>
    `}buildTip(){this.tipNeedsBuild=!1;let e=document.createElement("ed11y-element-tip"),t=(e.setAttribute("data-ed11y-result",this.resultID),document.querySelector("body"));t.insertAdjacentElement("beforeend",e),this.tip=e}toggleTip(e){this.tipNeedsBuild&&this.buildTip(),this.toggle.setAttribute("aria-expanded",e);var t=this.dismissable?"ed11y-ring-yellow":"ed11y-ring-red";this.result[0].classList.toggle(t),!0===e?(this.closeOtherTips(),this.allowOverflow(),this.tip.setAttribute("data-ed11y-action","open"),window.setTimeout(Ed11y.alignTip(this.toggle,this.tip)),Ed11y.elements.jumpList||Ed11y.buildJumpList(),Ed11y.goto=this.getAttribute("data-ed11y-jump-position"),Ed11y.setCurrentJump(),document.dispatchEvent(new CustomEvent("ed11yPop",{detail:{id:"ed11y-result-"+this.toggle.getAttribute("data-ed11y-result")}}))):(Ed11y.options.hiddenHandlers&&this.closest(Ed11y.options.hiddenHandlers)&&document.dispatchEvent(new CustomEvent("ed11yShut",{detail:{id:"ed11y-result-"+this.toggle.getAttribute("data-ed11y-result")}})),this.tip.setAttribute("data-ed11y-action","shut")),this.setAttribute("data-ed11y-open",e),this.open=e}handleHover(){"false"===this.getAttribute("data-ed11y-open")&&this.toggleTip(!0)}allowOverflow(){Ed11y.options.hiddenHandlers&&this.closest(Ed11y.options.hiddenHandlers)&&Ed11y.hiddenHandled!==this.getAttribute("id")&&document.dispatchEvent(new CustomEvent("ed11yShowHidden",{detail:{id:this.getAttribute("id")}}))}static get observedAttributes(){return["data-ed11y-action"]}attributeChangedCallback(e,t,i){this.initialized&&"data-ed11y-action"===e&&"false"!==i&&(e="open"===i,this.setAttribute("data-ed11y-action","false"),this.toggleTip(e))}}customElements.define("ed11y-element-result",Ed11yElementResult);class Ed11yElementTip extends HTMLElement{constructor(){super()}connectedCallback(){if(!this.initialized){this.open=!0,this.setAttribute("style","outline: 0px solid transparent;");const l=this.attachShadow({mode:"open"}),d=(this.resultID=this.dataset.ed11yResult,this.result=Ed11y.results[this.resultID],this.wrapper=document.createElement("div"),this.wrapper.setAttribute("role","dialog"),this.dismissable=!1!==this.result[4],this.wrapper.classList.add("wrapper"),this.wrapper.classList.add("ed11y-result"),this.addEventListener("mouseover",this.handleHover),document.createElement("style"));d.textContent=Ed11y.baseCSS+`
        :host {
          position: absolute;
          top: 10vh;
          left: 2vw;
          opacity: 0;
          transition: opacity .25s ease-in;
          z-index: 9998;
        }
        :host([data-ed11y-open='true']) {
          z-index: 9999;
          opacity: 1;
        }
        .wrapper {
          width: 1px;
          height: 1px;
          overflow: visible;
          color: ${Ed11y.color.text};
        }
        .arrow {
          display: none;
          content: "";
          position: absolute;
          transform: rotate(45deg);
          left: -10px;
          box-shadow: 0 0 0 2px ${Ed11y.color.bg}, 2px 2px 4px ${Ed11y.color.tipHeader}77;
          width: 20px;
          height: 20px;
          top: 6px;
        }
        .arrow[data-direction="left"] {
          left: -18px;
          background: linear-gradient(45deg, transparent 0%, transparent 48%, ${Ed11y.color.tipHeader} 49%);
        }
        .arrow[data-direction="under"] {
          margin-top: -18px;
          background: linear-gradient(-45deg, transparent 0%, transparent 48%, ${Ed11y.color.tipHeader} 49%);
        }
        .arrow[data-direction="above"] {
          margin-top: -27px;
          background: linear-gradient(135deg, transparent 0%, transparent 48%, ${Ed11y.color.tipHeader} 49%);
        }
        .arrow[data-direction="right"] {
          background: linear-gradient(-135deg, transparent 0%, transparent 48%, ${Ed11y.color.tipHeader} 49%);
        }
        .tip {
          font-size: 14px;
          z-index: 1;
          border: 2px solid ${Ed11y.color.tipHeader};
          background: ${Ed11y.color.bg};
          border-radius: 0 0 3px 3px;
          position:relative;
          width: clamp(12rem, 30rem, 89vw);
          display: none;
          box-shadow: 0 0 0 2px ${Ed11y.color.bg}, 2px 2px 4px ${Ed11y.color.primary}77;
        }
        @keyframes fade-in {
          0% { opacity: 0;}
          100% { opacity: 1;}
        }
        .open .tip {
          display: block;
        }
        .open .tip .content {
          animation: fade-in 0.25s ease-out;
        }
        .open .arrow {
          display: block;
          opacity: 1;
        }
        .title {
          background: ${Ed11y.color.tipHeader};
          color: ${Ed11y.color.primaryText};
          padding: 0 35px 1px 14px;
          font-weight: bold;
          font-size: 14px;
          min-height: 28px;
          line-height: 1;
          display: grid;
          place-content: center left;
          outline: transparent;
        }
        .content {
          padding: 0 12px 16px 16px;
        }
        p {
          margin-block-start: 1em;
          margin-block-end: 1em;
        }
        p:last-child {
          margin-block-end: 0;
        }
        button {
          margin: 0;
          border: 0;
          background: inherit;
          font-family: inherit;
          font-size: 11px;
          font-weight: 600;
          text-align: center;
          cursor: pointer;
          color: ${Ed11y.color.primaryText};
          background: ${Ed11y.color.tipHeader};
        }
        ul {
          margin-block-start: .643em;
          margin-block-end: .643em;
          padding-inline-start: 20px;
        }
        li {
          line-height: 1.357;
        }
        li + li {
          margin-top: .643em;
        }
        table {
          border-spacing: 0;
          margin: 20px;
        }
        th, td {
          border: 0;
          box-shadow: 0 0 0 1px;
          padding: 5px 10px;
        }
        a {
          color: inherit;
        }
        a:hover, a:focus-visible {
          text-decoration-style: double;
          text-decoration-skip-ink: none;
        }

        .close {
          padding: 0 0 0 2px;
          font-size: 14px;
          line-height: 1;
          height: 32px;
          display: grid;
          place-content: center;
          font-weight: 400;
          position: absolute;
          top: -3px;
          right: -2px;
          box-shadow: -1px 0 ${Ed11y.color.bg};
          background: transparent;
          width: 32px;
        }
        .close:hover {
          background: ${Ed11y.color.bg};
          color: ${Ed11y.color.text};
        }
        .dismiss {
          margin: .5em 1em .25em 0;
          padding: 5px 9px;
          border-radius: 3px;
          font-weight: bold;
        }
        .dismiss:hover, .dismiss:focus-visible {
          color: ${Ed11y.color.text};
          background: ${Ed11y.color.bg};
          box-shadow: inset 0 0 0 2px ${Ed11y.color.tipHeader};
        }
        .wrapper :focus-visible {
          outline: 2px solid transparent;
          box-shadow: inset 0 0 0 2px ${Ed11y.color.focusRing}, 0 0 0 3px ${Ed11y.color.primary};
        }
      `,this.tip=document.createElement("div"),this.tip.classList.add("tip"),this.tip.setAttribute("aria-labelledby","tip-title-"+[this.resultID]),this.heading=document.createElement("div"),this.heading.setAttribute("id","tip-"+this.resultID),this.heading.classList.add("title"),this.heading.setAttribute("tabindex","-1"),this.heading.innerHTML=Ed11y.M[this.result[1]].title;let i=document.createElement("div");if(i.classList.add("content"),i.innerHTML=this.result[2],this.dismissable&&(Ed11y.options.allowOK||Ed11y.options.allowHide)){let t=document.createElement("div");if(Ed11y.options.allowOK){let e=document.createElement("button");e.classList.add("dismiss"),e.textContent="Mark as Checked and OK",e.setAttribute("title","Dismisses alert for all editors"),t.append(e),e.addEventListener("click",function(){Ed11y.dismissThis("ok")})}if(Ed11y.options.allowHide){let e=document.createElement("button");e.classList.add("dismiss"),e.textContent="Hide alert",e.setAttribute("title","Dismisses alert for you"),t.append(e),e.addEventListener("click",function(){Ed11y.dismissThis("hide")})}let e=document.createElement("button");e.classList.add("dismiss"),e.textContent="?",t.append(e),e.addEventListener("click",function(){Ed11y.dismissHelp(e)}),i.append(t)}let e=document.createElement("button"),t=(e.setAttribute("aria-label","close"),e.classList.add("close"),e.innerHTML="&times;",document.createElement("div")),n=(t.classList.add("arrow"),this.tip.append(this.heading),this.tip.append(e),this.tip.append(i),e.addEventListener("click",e=>{if(e.preventDefault,this.open){let e=document.querySelector('ed11y-element-result[data-ed11y-open="true"]');e?.shadowRoot.querySelector("button").focus(),e?.setAttribute("data-ed11y-action","shut"),this.setAttribute("data-ed11y-action","shut")}}),l.appendChild(d),l.appendChild(this.wrapper),document.createElement("div")),s=(n.setAttribute("tabIndex","0"),document.createElement("div")),a=(s.setAttribute("tabindex","0"),this.wrapper.appendChild(n),this.wrapper.appendChild(t),this.wrapper.appendChild(this.tip),this.wrapper.appendChild(s),this.wrapper.querySelectorAll('a, button, [tabindex="0"]')),o=a.length;a[0].addEventListener("focus",()=>{a[o-2].focus()}),a[o-1].addEventListener("focus",()=>{a[1].focus()}),this.initialized=!0}}toggleTip(e){e?this.wrapper.classList.add("open"):this.wrapper.classList.remove("open"),this.setAttribute("data-ed11y-open",e)}toggleImage(){var e=`
      .toggle {
        box-shadow: inset 0 0 0 2px ${Ed11y.color.warning}, inset 0 0 0 3px #444, inset 0 0 0 6px ${Ed11y.color.warning}, 1px 1px 5px 0 rgba(0,0,0,.5);
        background: ${Ed11y.color.warning};
        color: #333;
      }
      .toggle::before {
        content: "?";
      }
      .toggle:hover, .toggle[aria-expanded='true'] {
        border: 2px solid ${Ed11y.color.primary};
      }`,t=`
      .toggle {
        box-shadow: inset 0 0 0 1px ${Ed11y.color.alert}, inset 0 0 0 2px #fefefe, inset 0 0 0 6px #b80519, 1px 1px 5px 0 rgba(0,0,0,.5);
        background: #fefefe;
        color: ${Ed11y.color.alert};
      }
      .toggle:hover, .toggle[aria-expanded='true'] {
        box-shadow: inset 0 0 0 1px ${Ed11y.color.alert}, inset 0 0 0 2px #fefefe, inset 0 0 0 6px #b80519, 0 0 0 2px ${Ed11y.color.primary}, 0 0 0 3px transparent;
      }
      .toggle::before {
        content: "!";
      }`;return this.dismissable?e:t}tipDOM(e,t,i){return`>
      <div class="title" id="tip-title-${e}">${t}</div>
      <div class="message">${i}</div>
    `}static get observedAttributes(){return["data-ed11y-action"]}attributeChangedCallback(e,t,i){this.initialized&&"data-ed11y-action"===e&&"false"!==i&&(this.open=e="open"===i,this.setAttribute("data-ed11y-action","false"),this.toggleTip(e))}}customElements.define("ed11y-element-tip",Ed11yElementTip);class Ed11yTestEmbeds{check(){let e=Ed11y.srcMatchesOptions(Ed11y.elements.iframe,Ed11y.options.videoContent),t=(0<(e=e.concat(Array.from(Ed11y.elements.video))).length&&e.forEach(e=>{var t=Ed11y.dismissalKey("undefined"!==e?.getAttribute("src")?e.getAttribute("src"):e.querySelector("[src]")?.getAttribute("src"));Ed11y.results.push([e,"embedVideo",Ed11y.M.embedVideo.tip(),"beforebegin",t])}),Ed11y.srcMatchesOptions(Ed11y.elements.iframe,Ed11y.options.audioContent)),i=(0<(t=t.concat(Array.from(Ed11y.elements.audio))).length&&t.forEach(e=>{var t=Ed11y.dismissalKey("undefined"!==e?.getAttribute("src")?e.getAttribute("src"):e.querySelector("[src]")?.getAttribute("src"));Ed11y.results.push([e,"embedAudio",Ed11y.M.embedAudio.tip(),"beforebegin",t])}),Ed11y.srcMatchesOptions(Ed11y.elements.iframe,Ed11y.options.dataVizContent)),n=(0<i.length&&i.forEach(e=>{var t=Ed11y.dismissalKey(e.getAttribute("src"));Ed11y.results.push([e,"embedVisualization",Ed11y.M.embedVisualization.tip(),"beforebegin",t])}),Ed11y.srcMatchesOptions(Ed11y.elements.frame,Ed11y.options.twitterContent));0<n.length&&n.forEach(e=>{var t=e.querySelectorAll(".timeline-TweetList-tweet");t&&3<t&&(t=Ed11y.dismissalKey(e.getAttribute("src")),Ed11y.results.push([e,"embedTwitter",Ed11y.M.embedTwitter.tip(),"beforebegin",t]))}),Ed11y.options.embeddedContent&&Ed11y.elements.embed?.forEach(e=>{var t=Ed11y.dismissalKey(e.getAttribute("src")+e.getAttribute("id")+e.getAttribute("class"));Ed11y.results.push([e,"embedCustom",Ed11y.M.embedCustom.tip(),"beforebegin",t])})}}class Ed11yTestHeadings{check(){Ed11y.panel?.querySelectorAll("#ed11y-outline-list li, .ed11y-headings-label")?.forEach(e=>e.remove());let r=0,p=(Ed11y.headingOutline=[],"beforebegin");Ed11y.elements.h?.forEach((e,t)=>{let i,n=(i=e.hasAttribute("aria-level")?+e.getAttribute("aria-level"):+e.tagName.slice(1),i=parseInt(i),""),s=!1,a="";var o=Ed11y.getText(e),l=o.length;let d=!1;1<i-r&&0!==t&&(n="headingLevelSkipped",d=i+o,s="(flagged for skipped level) ",a=Ed11y.M.headingLevelSkipped.tip(r,i)),l<1?(t=e.querySelector("img")?.getAttribute("alt"))&&0!==t.length||(n="headingEmpty",s="(empty heading)",a=Ed11y.M.headingEmpty.tip()):160<l&&(s="(flagged for length) ",d=i+o,n="headingIsLong",a=Ed11y.M.headingIsLong.tip()),r=i,""!==n&&(s=e.closest(Ed11y.options.checkRoots)||Ed11y.options.shadowComponents&&e.getRootNode()?.host?.matches(Ed11y.options.shadowComponents)?(Ed11y.results.push([e,n,a,p,d]),s&&"<span class='ed11y-small'><em>"+s+"</em></span>"):""),Ed11y.headingOutline.push([e,i,s])}),Ed11y.elements.blockquote?.forEach(e=>{var t,i=Ed11y.getText(e);i.length<25&&(i=Ed11y.dismissalKey(i),t=Ed11y.M.blockquoteIsShort.tip(),Ed11y.results.push([e,"blockquoteIsShort",t,p,i]))})}}class Ed11yTestImages{check(){Ed11y.imageAlts=[],Ed11y.elements.img.forEach(n=>{let s=n.getAttribute("alt"),a="Alt text: ",e=n.getAttribute("src"),o=!1,l=!0;var t,d=Ed11y.parentLink(n);if("string"!=typeof s)o="altMissing",l=!1,a+="(missing!)";else if(0!==s.length||d){a+=s;let e=[".png",".jpg",".jpeg",".gif"],t=["image of","graphic of","picture of","placeholder","photo of"],i=[null,null];e.forEach(e=>{0<=s.toLowerCase().indexOf(e)&&(i[0]="URL")}),t.forEach(e=>{0<=s.toLowerCase().indexOf(e)&&(i[1]=e)}),"URL"===i[0]?(o="altURL",l=!1):null!==i[1]?o="altImageOf":""!==s&&""===s.replace(/"|'|\s+/g,"")?(o="altDeadspace",l=!1):160<s.length&&(o="altLong"),d&&(n=d,o=!o&&1<Ed11y.linkText(d.textContent).length?"altPartOfLinkWithText":o&&o+"Linked")}else o="altNull",a+="(none; image marked as decorative)";let i="pass";o&&(d=Ed11y.M[o].tip(Ed11y.sanitizeForHTML(s)),t=e.split("?")[0],l=!!l&&Ed11y.dismissalKey(t+s),Ed11y.results.push([n,o,d,"beforebegin",l]),i=!1===l?"error":"warning"),Ed11y.imageAlts.push([n,e,a,i])})}}class Ed11yTestLinks{check(){Ed11y.elements.a?.forEach(i=>{let n=Ed11y.computeAriaLabel(i),e=i.querySelectorAll("img"),s=0<e.length,t=!1,a=(i.matches(Ed11y.options.documentLinks)&&(t=!0),"noAria"===n&&(n=Ed11y.getText(i),s&&(e.forEach(e=>{var t=Ed11y.computeAriaLabel(e);"noAria"!==t?n+=t:e.hasAttribute("alt")?n+=e.getAttribute("alt"):e.hasAttribute("src")&&(n+=e.getAttribute("src"))}),s=!0)),n+=Ed11y.computeTitle(i)?Ed11y.computeTitle(i):"",Ed11y.options.linkIgnoreStrings?n.replace(Ed11y.options.linkIgnoreStrings,""):n);if("_blank"===i?.getAttribute("target")&&n===a&&-1===a.indexOf("window")&&-1===a.indexOf("tab")&&(o=Ed11y.dismissalKey(n),Ed11y.results.push([i,"linkNewWindow",Ed11y.M.linkNewWindow.tip(),"beforebegin",o])),0===(a=a.replace(/'|"|-|\.|\s+/g,"").toLowerCase()).length)!1===s?Ed11y.results.push([i,"linkNoText",Ed11y.M.linkNoText.tip(),"beforebegin",!1]):Ed11y.results.push([i,"altEmptyLinked",Ed11y.M.altEmptyLinked.tip(),"beforebegin",!1]);else{var o=function(t){var i=["http:/","https:/",".asp",".htm",".php",".edu/",".com/"];let n="none";if(0===t.replace(/learn|to|more|now|this|page|link|site|website|check|out|view|our|read|\.|,|:|download|form|here|click|>|<|\s/g,"").length)n="generic";else for(let e=0;e<i.length;e++)if(-1<t.indexOf(i[e])){n="url";break}return n}(a);if("none"!==o){let e=!1,t=!1;s||(t="linkTextIsURL",e=Ed11y.dismissalKey(n)),"generic"===o&&(t="linkTextIsGeneric",e=Ed11y.dismissalKey(n)),t&&Ed11y.results.push([i,t,Ed11y.M[t].tip(Ed11y.sanitizeForHTML(n)),"beforebegin",e])}}!s&&t&&(o=Ed11y.dismissalKey(i?.getAttribute("src")),Ed11y.results.push([i,"linkDocument",Ed11y.M.linkDocument.tip(),"beforebegin",o]))})}}class Ed11yTestText{check(){function a(e){return e.replace(/^b|^B|^2/,function(e){return i[e]})}function t(e){let t="";"LI"===e.tagName?e.childNodes.forEach(e=>{3===e.nodeType&&(t+=e.textContent)}):t=Ed11y.getText(e);var i,n=t.match(/([A-Z]{2,}[ ])([A-Z]{2,}[ ])([A-Z]{2,}[ ])([A-Z]{2,})/g);n&&10<n[0].length&&(n=Ed11y.dismissalKey(t),(i=e.closest("a, button"))?Ed11y.results.push([i,"textUppercase",Ed11y.M.textUppercase.tip(),"beforebegin",n]):Ed11y.results.push([e,"textUppercase",Ed11y.M.textUppercase.tip(),"afterbegin",n]))}let o="",l="",i={b:"a",B:"A",2:"1"},d=/a\.|a\)|A\.|A\)|1\.|1\)|\*\s|-\s|--|•\s|→\s|✓\s|✔\s|✗\s|✖\s|✘\s|❯\s|›\s|»\s/;Ed11y.elements.p?.forEach((t,i)=>{let n="",e=!1,s=!1;if(!l){let e=Ed11y.getText(t);n=e.substring(0,2)}if(0<n.length&&n!==o&&n.match(d)){var i=Ed11y.elements.p[i+1];if(i&&(e=Ed11y.getText(i).substring(0,2),i=a(e),n===i&&(s=!0)),!s){let e=t?.querySelector("br")?.nextSibling?.nodeValue;e&&(e=e.replace(/<\/?[^>]+(>|$)/g,"").trim().substring(0,2),n===a(e)&&(s=!0))}o=s?(i=Ed11y.dismissalKey(l),Ed11y.results.push([t,"textPossibleList",Ed11y.M.textPossibleList.tip(n),"afterbegin",i]),n):""}else{let e=t.querySelector("strong, b");e&&(i=null!==(e=Ed11y.getText(e)).match(/[.:;?!"']/),0<(e.length<121)&&!1==i&&(i=Ed11y.dismissalKey(e),Ed11y.results.push([t,"textPossibleHeading",Ed11y.M.textPossibleHeading.tip(),"afterbegin",i])))}l=e&&""});Ed11y.elements.h?.forEach(e=>{t(e)}),Ed11y.elements.p?.forEach(e=>{t(e)}),Ed11y.elements.blockquote?.forEach(e=>{t(e)}),Ed11y.elements.li?.forEach(e=>{t(e)}),Ed11y.elements.table.forEach(e=>{let t=e.querySelectorAll("th"),i=e.querySelectorAll("h1, h2, h3, h4, h5, h6");0===t.length?Ed11y.results.push([e,"tableNoHeaderCells",Ed11y.M.tableNoHeaderCells.tip(),"beforebegin",!1]):t.forEach(e=>{Ed11y.getText(e).length<1&&!Ed11y.computeTitle(e)&&Ed11y.results.push([e,"tableEmptyHeaderCell",Ed11y.M.tableEmptyHeaderCell.tip(),"afterbegin",!1])}),i&&i.forEach(e=>{Ed11y.results.push([e,"tableContainsContentHeading",Ed11y.M.tableContainsContentHeading.tip(),"beforebegin",!1])})})}}class Ed11y{constructor(e){Ed11y.options={checkRoots:!1,shadowComponents:!1,ignoreElements:!1,ignoreByKey:{img:"[aria-hidden], [aria-label], [aria-labelledby], [aria-hidden] img, [aria-label] img, [aria-labelledby] img",a:"[aria-hidden][tabindex]"},alertMode:"polite",currentPage:!1,allowHide:!0,allowOK:!0,syncedDismissals:!1,ignoreAllIfAbsent:!1,ignoreAllIfPresent:!1,preventCheckingIfPresent:!1,preventCheckingIfAbsent:!1,linkIgnoreStrings:!1,hiddenHandlers:"",lang:"en",theme:"lightTheme",darkTheme:{bg:"#0a2051",bgHighlight:"#7b1919",text:"#f4f7ff",primary:"#3052a0",primaryText:"#f4f7ff",secondary:"#20160c",warning:"#fad859",alert:"#b80519",button:"#dde8ff",focusRing:"cyan",activeTab:"#0a2051",tipHeader:"#3052a0"},lightTheme:{bg:"#fffffe",bgHighlight:"#7b1919",text:"#20160c",primary:"#0a307a",primaryText:"#fffdf7",secondary:"#20160c",warning:"#fad859",alert:"#b80519",button:"#0a307a",focusRing:"#007aff",activeTab:"#b9c0cf",tipHeader:"#0a307a"},embeddedContent:!1,embeddedContentTitle:"",embeddedContentMessage:"",videoContent:"youtube.com, vimeo.com, yuja.com, panopto.com",audioContent:"soundcloud.com, simplecast.com, podbean.com, buzzsprout.com, blubrry.com, transistor.fm, fusebox.fm, libsyn.com",dataVizContent:"datastudio.google.com, tableau",twitterContent:"twitter-timeline",documentLinks:"a[href$='.pdf'], a[href*='.pdf?'], a[href$='.doc'], a[href$='.docx'], a[href*='.doc?'], a[href*='.docx?'], a[href$='.ppt'], a[href$='.pptx'], a[href*='.ppt?'], a[href*='.pptx?'], a[href^='https://docs.google']",...e},Ed11y.M=ed11yLang[Ed11y.options.lang],Ed11y.color=Ed11y.options[Ed11y.options.theme],!1===Ed11y.options.currentPage&&(Ed11y.options.currentPage=window.location.pathname),Ed11y.initialize=()=>{Ed11y.checkRunPrevent=()=>{let e=!!Ed11y.options.preventCheckingIfPresent&&document.querySelector(Ed11y.options.preventCheckingIfPresent);return e=!e&&Ed11y.options.preventCheckingIfAbsent?null===document.querySelector(`:is(${Ed11y.options.preventCheckingIfAbsent})`):e};var e;e=()=>{Ed11y.checkRunPrevent()||(Ed11y.running=!0,Ed11y.localData=localStorage.getItem("Ed11y.localData"),Ed11y.localDataParsed=Ed11y.localData?JSON.parse(Ed11y.localData):{},!1===Ed11y.options.syncedDismissals?(Ed11y.dismissedAlerts=localStorage.getItem("ed11ydismissed"),Ed11y.dismissedAlerts=Ed11y.dismissedAlerts?JSON.parse(Ed11y.dismissedAlerts):{}):(Ed11y.dismissedAlerts={},Ed11y.dismissedAlerts[Ed11y.options.currentPage]=Ed11y.options.syncedDismissals),Ed11y.testEmbeds=new Ed11yTestEmbeds,Ed11y.testHeadings=new Ed11yTestHeadings,Ed11y.testImages=new Ed11yTestImages,Ed11y.testLinks=new Ed11yTestLinks,Ed11y.testText=new Ed11yTestText,Ed11y.ignore=Ed11y.options.ignoreElements?`:not(${Ed11y.options.ignoreElements})`:"",Ed11y.options.checkRoots||(Ed11y.options.checkRoots=null!==document.querySelector("main")?"main":"body"),Ed11y.ignoreAll=!(!Ed11y.options.ignoreAllIfAbsent||null!==document.querySelector(`:is(${Ed11y.options.ignoreAllIfAbsent})`)),!Ed11y.ignoreAll&&Ed11y.options.ignoreAllIfPresent&&(Ed11y.ignoreAll=null!==document.querySelector(`:is(${Ed11y.options.ignoreAllIfPresent})`)),Ed11y.checkAll(!0,"hide"))},"complete"===document.readyState?e():window.addEventListener("load",e)},Ed11y.checkAll=(e,i)=>{if(Ed11y.checkRunPrevent())Ed11y.reset(),Ed11y.panelToggle.classList.add("disabled"),Ed11y.panelToggle.setAttribute("aria-expanded","true"),Ed11y.panelToggle.setAttribute("title","Editorially is disabled.");else{Ed11y.results=[],Ed11y.elements=[],Ed11y.errorCount=0,Ed11y.warningCount=0,Ed11y.dismissedCount=0,Ed11y.mediaCount=0;let t=document.querySelectorAll(`:is(${Ed11y.options.checkRoots})`);if(0===t.length)Ed11y.roots=[document.querySelector("html, body")],Ed11y.elements=[],0===t.length?console.warn("Check Editoria11y configuration; specified root element not found"):console.warn("No user editable content on this page"),Ed11y.reset();else{Ed11y.roots=[],t.forEach((e,t)=>{e.shadowRoot?Ed11y.roots[t]=e.shadowRoot:Ed11y.roots[t]=e}),Ed11y.buildElementList();let e=["testLinks","testImages","testHeadings","testText","testEmbeds"];e.forEach(e=>{window.setTimeout(function(){Ed11y[e].check()},0,e)})}window.setTimeout(function(){Ed11y.updatePanel(e,i),Ed11y.panelToggle.setAttribute("title","Toggle accessibility tools")},0)}},Ed11y.countAlerts=function(e){if(Ed11y.ignoreAll)Ed11y.dismissedCount=0<Ed11y.results.length?1:0;else{Ed11y.dismissedCount=0;for(let e=Ed11y.results.length-1;0<=e;e--){var t=Ed11y.results[e][1],i=Ed11y.results[e][4];!1!==i&&void 0!==Ed11y.dismissedAlerts[Ed11y.options.currentPage]&&void 0!==Ed11y.dismissedAlerts[Ed11y.options.currentPage][t]&&"undefined"!==Ed11y.dismissedAlerts[Ed11y.options.currentPage][t][i]?(Ed11y.dismissedCount++,Ed11y.results[e][5]=Ed11y.dismissedAlerts[Ed11y.options.currentPage][t][i]):(Ed11y.results[e][4]?Ed11y.warningCount++:Ed11y.errorCount++,Ed11y.results[e][5]=!1)}}Ed11y.totalCount=Ed11y.errorCount+Ed11y.warningCount,0<Ed11y.totalCount?(Ed11y.panelJumpNext.removeAttribute("hidden"),Ed11y.totalCount<2?Ed11y.panelJumpPrev.setAttribute("hidden",""):Ed11y.panelJumpPrev.removeAttribute("hidden"),0<Ed11y.errorCount?(Ed11y.panel.classList.remove("warnings","pass"),Ed11y.panel.classList.add("errors")):0<Ed11y.warningCount&&(Ed11y.panel.classList.remove("errors","pass"),Ed11y.panel.classList.add("warnings")),Ed11y.panelCount.textCount=Ed11y.totalCount,Ed11y.panelCount.style.display="inline-block",Ed11y.panelMessage.innerHTML=1===Ed11y.totalCount?Ed11y.M.panelCount1:Ed11y.totalCount+Ed11y.M.panelCountMultiple,Ed11y.panel.querySelector(".toggle-count").textContent=Ed11y.totalCount):(Ed11y.panelJumpNext.setAttribute("hidden",""),Ed11y.panelJumpPrev.setAttribute("hidden",""),Ed11y.panelCount.style.display="display: none;",Ed11y.panel.classList.remove("warnings","errors"),Ed11y.panel.classList.add("pass"),0<Ed11y.dismissedCount?(Ed11y.panel.querySelector(".toggle-count").textContent="i",Ed11y.panelMessage.innerText=Ed11y.M.panelCountAllDismissed):(Ed11y.panel.querySelector(".toggle-count").textContent="✓",Ed11y.panelMessage.innerText=Ed11y.M.panelCount0)),(0<Ed11y.dismissedCount||Ed11y.ignoreAll)&&Ed11y.restoreDismissed.removeAttribute("hidden"),Ed11y.panel.classList.remove("ed11y-preload"),!0===e&&window.setTimeout(function(){var e=new CustomEvent("ed11yResults");document.dispatchEvent(e)},0)},Ed11y.updatePanel=function(e,t){var i;!0===e&&(i=document.createElement("ed11y-element-panel"),document.querySelector("body").appendChild(i)),Ed11y.countAlerts(e),!0===e&&0<Ed11y.totalCount&&!Ed11y.ignoreAll?(Ed11y.localDataParsed[Ed11y.options.currentPage]===Ed11y.totalCount?t="seen":"assertive"===Ed11y.options.alertMode&&(t="show"),Ed11y.localDataParsed[Ed11y.options.currentPage]=Ed11y.totalCount,window.setTimeout(function(){Ed11y.announce.innerHTML=Ed11y.getText(Ed11y.panelMessage)},1500)):!0===e&&0===Ed11y.totalCount&&(t="pass"),"show"!==t?(Ed11y.reset(),Ed11y.bodyStyle||Ed11y.paintReady()):(Ed11y.panel.classList.remove("shut"),Ed11y.panel.classList.add("active"),Ed11y.panelToggle.setAttribute("aria-expanded","true"),window.setTimeout(function(){document.dispatchEvent(new CustomEvent("ed11yPanelOpened")),Ed11y.ignoreAll||Ed11y.showResults()},0),!1===e&&window.setTimeout(function(){Ed11y.panelMessage.focus()},500)),Ed11y.panelToggle.classList.remove("disabled"),Ed11y.panelToggle.removeAttribute("aria-disabled"),Ed11y.running=!1},Ed11y.result=function(e,t){let i=document.createElement("ed11y-element-result"),n=e[0].closest("a"),s="beforebegin";n||(n=e[0],s=e[3]),i.setAttribute("id","ed11y-result-"+t),i.setAttribute("data-ed11y-result",t),i.setAttribute("data-ed11y-open","false"),n.insertAdjacentElement(s,i)},Ed11y.reset=function(){Ed11y.resetClass(["ed11y-ring-red","ed11y-ring-yellow","ed11y-hidden-highlight"]),Ed11y.findElements("reset","ed11y-element-result, ed11y-element-tip, .ed11y-element-heading-label, .ed11y-element-alt, ed11y-element-heading-label, ed11y-element-alt",!1),Ed11y.elements.reset.forEach(e=>e.remove()),Ed11y.panelJumpNext?.setAttribute("data-ed11y-goto","0"),Ed11y.panelJumpPrev?.setAttribute("data-ed11y-goto","0"),Ed11y.panel?.classList.add("shut"),Ed11y.panel?.classList.remove("active"),Ed11y.panelToggle?.setAttribute("aria-expanded","false"),Ed11y.running=!1},Ed11y.linkText=e=>e=(e=e.replace(Ed11y.options.linkIgnoreStrings,"")).replace(/'|"|-|\.|\s+/g,""),Ed11y.computeAriaLabel=function(e,t=!1){var i=e.hasAttribute("aria-label"),n=e.hasAttribute("aria-labelledby");if(!t){if(n){let t=e.getAttribute("aria-labelledby"),i="";if(0<t.length){t=(t="#"+t).replace(/ /g,", #");let e=document.querySelectorAll(t);e?.forEach(e=>{i+=Ed11y.computeAriaLabel(e,!0)+" "})}return i}if(!i)return"noAria"}return i?e.getAttribute("aria-label"):t?Ed11y.getText(e):void 0},Ed11y.computeTitle=function(e){return e.hasAttribute("title")?e.getAttribute("title"):e.querySelector("[title]")?e.querySelector("[title]").getAttribute("title"):""},Ed11y.findElements=function(t,n,e=!0){let i=Ed11y.options.shadowComponents?", "+Ed11y.options.shadowComponents:"",s="";if(s=Ed11y.options.ignoreElements?Ed11y.options.ignoreByKey[t]?`:not(${Ed11y.options.ignoreElements}, ${Ed11y.options.ignoreByKey[t]})`:`:not(${Ed11y.options.ignoreElements})`:Ed11y.options.ignoreByKey[t]?`:not(${Ed11y.options.ignoreByKey[t]})`:"",Ed11y.elements[t]=[],e?Ed11y.roots.forEach(e=>{Ed11y.elements[t]=Ed11y.elements[t].concat(Array.from(e.querySelectorAll(`:is(${n}${i})`+s)))}):Ed11y.elements[t]=Ed11y.elements[t].concat(Array.from(document.querySelectorAll(`:is(${n}${i})`+s))),Ed11y.options.shadowComponents){let i=[];if(Ed11y.elements[t].forEach((e,t)=>{e.matches(Ed11y.options.shadowComponents)&&(i[t]=e.shadowRoot.querySelectorAll(`:is(${n})`+s))}),0<i.length)for(let e=i.length-1;0<=e;e--)i[e]&&Ed11y.elements[t].splice(e,1,...i[e])}},Ed11y.buildElementList=function(){Ed11y.findElements("p","p"),Ed11y.findElements("h",'h1, h2, h3, h4, h5, h6, [role="heading"][aria-level]',!1),Ed11y.findElements("img","img"),Ed11y.findElements("a","a[href]"),Ed11y.findElements("li","li"),Ed11y.findElements("blockquote","blockquote"),Ed11y.findElements("iframe","iframe"),Ed11y.findElements("audio","audio"),Ed11y.findElements("video","video"),Ed11y.findElements("table","table"),Ed11y.options.embeddedContent&&Ed11y.findElements("embed",Ed11y.options.embeddedContent)},Ed11y.dismissalKey=function(e){return String(e).substring(0,512)},Ed11y.dismissThis=function(e){let t=Ed11y.getOpenTip();var i=t.dataset.ed11yResult,n=Ed11y.results[i][1],i=Ed11y.dismissalKey(Ed11y.results[i][4]);Ed11y.resetClass(["ed11y-hidden-highlight","ed11y-ring-red","ed11y-ring-yellow"]),t.parentNode.removeChild(t);let s=Ed11y.elements.openButton[0],a=(s.parentNode.removeChild(s),{});if(a[i]=e,void 0===Ed11y.dismissedAlerts[Ed11y.options.currentPage]){let e={};e[n]=a,Ed11y.dismissedAlerts[Ed11y.options.currentPage]=e}else void 0===Ed11y.dismissedAlerts[Ed11y.options.currentPage][n]?Ed11y.dismissedAlerts[Ed11y.options.currentPage][n]=a:Ed11y.dismissedAlerts[Ed11y.options.currentPage][n][i]=e;!1===Ed11y.options.syncedDismissals?localStorage.setItem("ed11ydismissed",JSON.stringify(Ed11y.dismissedAlerts)):(n={dismissPage:Ed11y.options.currentPage,dismissTest:n,dismissKey:i,dismissAction:e},i=new CustomEvent("ed11yDismissalUpdate",{detail:n}),document.dispatchEvent(i)),Ed11y.reset(),Ed11y.checkAll(!1,"show");let o=Ed11y.goto;window.setTimeout(function(){Ed11y.buildJumpList(),0<Ed11y.elements.jumpList.length?(Ed11y.goto=o-1,Ed11y.setCurrentJump(),Ed11y.panelJumpNext.focus()):window.setTimeout(function(){let e=Ed11y.panel.querySelector("#issues-tab");e.focus()},100)},500,o)},Ed11y.clearDismissals=function(){var e;Ed11y.ignoreAll=!1,!(Ed11y.dismissedAlerts[Ed11y.options.currentPage]={})===Ed11y.options.syncedDismissals?localStorage.setItem("ed11ydismissed",JSON.stringify(Ed11y.dismissedAlerts)):(e={dismissPage:Ed11y.options.currentPage,dismissAction:"reset"},e=new CustomEvent("ed11yDismissalUpdate",{detail:e}),document.dispatchEvent(e)),Ed11y.restoreDismissed.setAttribute("hidden",""),Ed11y.reset(),Ed11y.checkAll(!1,"show")},Ed11y.dismissHelp=function(e){let t=document.createElement("ul");if(t.setAttribute("tabindex","-1"),t.classList.add("help"),Ed11y.options.allowHide){let e=document.createElement("li");e.textContent=Ed11y.M.elementDismissalHelpHide,t.append(e)}if(Ed11y.options.allowOK){let e=document.createElement("li");e.textContent=Ed11y.M.elementDismissalHelpOK,t.append(e)}let i=document.createElement("li");i.textContent=Ed11y.M.elementDismissalHelpAll,t.append(i),e.parentElement.append(t),t.focus(),e.remove()},Ed11y.showResults=function(){Ed11y.results?.forEach(function(e,t){Ed11y.results[t][5]||Ed11y.result(e,t)}),Ed11y.buildJumpList(),document.dispatchEvent(new CustomEvent("ed11yPanelOpened")),Ed11y.alignTips()},Ed11y.alignTips=function(){window.setTimeout(function(){let i=window.innerWidth,n=[];Ed11y.elements.jumpList.forEach(e=>{e.hasAttribute("style")&&e.removeAttribute("style");var t=e.getBoundingClientRect();t.left<8?n.push([e,8-t.left]):t.left+80>i&&n.push([e,i-t.left-80])}),n.forEach(e=>{e[0].style.transform="translate("+e[1]+"px, 0)"}),Ed11y.bodyStyle||Ed11y.paintReady()},0)},Ed11y.paintReady=function(){let i=`ed11y-element-result, ed11y-element-panel {
              opacity: 1; 
              outline: 0 !important;
            }
            ed11y-element-result + ed11y-element-result {
              margin-top: 34px;
            }
            .ed11y-hidden-highlight {
              box-shadow: inset 0 0 0 1px ${Ed11y.color.warning}, inset 0 0 0 2px ${Ed11y.color.primary}, 0 0 0 1px ${Ed11y.color.warning}, 0 0 0 3px ${Ed11y.color.primary}, 0 0 1px 3px !important;
            }
            .ed11y-ring-red {
              box-shadow: 0 0 0 1px #fff, inset 0 0 0 2px ${Ed11y.color.alert}, 0 0 0 3px ${Ed11y.color.alert}, 0 0 1px 3px;
              outline: 2px solid ${Ed11y.color.alert};
              outline-offset: 1px;
            }
            .ed11y-ring-yellow {
              box-shadow: 0 0 0 1px #fff, inset 0 0 0 2px ${Ed11y.color.warning}, 0 0 0 3px ${Ed11y.color.warning}, 0 0 1px 3px;
              outline: 2px solid ${Ed11y.color.warning};
              outline-offset: 1px;
            }
            `;Ed11y.roots.forEach(e=>{let t=document.createElement("style");t.textContent=i,e.appendChild(t),Ed11y.options.shadowComponents&&e.querySelectorAll(Ed11y.options.shadowComponents).forEach(e=>{let t=document.createElement("style");t.textContent=i,e.shadowRoot.appendChild(t)})}),Ed11y.roots.forEach(e=>{var t=document.createElement("style");e.appendChild(t)}),Ed11y.bodyStyle=!0},Ed11y.alignTip=function(t,e){let i=e.shadowRoot.querySelector(".arrow"),n=i.nextElementSibling;n.setAttribute("style",""),i.setAttribute("style","");var s=window.scrollY;let a=t.getBoundingClientRect();if(0===a.top&&0===a.left){let e=Ed11y.firstVisibleParent(t.getRootNode().host);e&&(a=e.getBoundingClientRect())}var t=a.left+document.body.scrollLeft,o=(e.style.setProperty("top",a.top+s+"px"),e.style.setProperty("left",a.left+"px"),n.offsetWidth),l=n.offsetHeight,d=window.innerWidth,r=s+window.innerHeight;let p="under",c=(a.top+l+s+50>r&&(0<d-(t+o+90)&&a.top+40<window.innerHeight?p="right":a.top>l+15?p="above":0<t-o-50&&(p="left")),0),u=0;"under"===p?(4*o/5+a.left+20>d||a.left-20-o/5<0?o-15>a.left?(c=15-a.left,i.style.setProperty("left",Math.max(a.left,15)-9+"px")):(i.style.setProperty("left",o-26+"px"),c=31-o):(c=40-o/5,i.style.setProperty("left",o/5-34+"px")),i.dataset.direction="under",u=50):"above"===p?(u=-1*(l+18),i.style.setProperty("top",l),4*o/5+a.left+20>d||a.left-20-o/5<0?o-15>a.left?(c=15-a.left,i.style.setProperty("left",a.left-9+"px")):(i.style.setProperty("left",o-26+"px"),c=31-o):(c=40-o/5,i.style.setProperty("left",o/5-34+"px")),i.dataset.direction="above",i.style.setProperty("top",l+18+"px")):(r<(t=a.top+s+l)&&(d=-1*(u=r-(t+10))+7,d=Math.min(d,l-25),i.style.setProperty("top",d+"px")),"left"===p?(c=0-(o+17),i.style.setProperty("left",o-9+"px"),i.dataset.direction="left"):(c=50,i.dataset.direction="right")),e.style.setProperty("transform",`translate(${c}px, ${u}px)`)},Ed11y.baseCSS=`
      :host{all: initial;} 
      .hidden{display:none;}
      .wrapper{font-size:14px;line-height: 1.5;font-family:-apple-system,BlinkMacSystemFont,"Segoe UI",Roboto,"Helvetica Neue",Arial,sans-serif;}
      `,Ed11y.clickTab=function(e){Ed11y.activateTab(e.target,!1)},Ed11y.togglePanel=function(){return Ed11y.doubleClickPrevent||(Ed11y.panel.classList.contains("active")&&Ed11y.panel.classList.contains("shut")?Ed11y.minimize():!0!==Ed11y.running&&(!(Ed11y.running=!0)===Ed11y.panel.classList.contains("active")?Ed11y.checkAll(!1,"show"):Ed11y.reset())),Ed11y.doubleClickPrevent=!0,window.setTimeout(function(){Ed11y.doubleClickPrevent=!1},200),!1},Ed11y.showHeadingsPanel=function(){let l=Ed11y.panel.querySelector("#outline");Ed11y.headingOutline.length?(l.innerHTML="",Ed11y.headingOutline.forEach((e,t)=>{let i=document.createElement("ed11y-element-heading-label");i.dataset.ed11yHeadingOutline=t,e[0].insertAdjacentElement("afterbegin",i);var t=e[1],n=10*t-10;let s=document.createElement("li");s.classList.add("level"+t);var a=e[2]&&!e[5]?e[2]:"";a&&s.classList.add("has-issues"),s.style.setProperty("margin-left",n+"px"),s.innerHTML=`<strong>H${t}:</strong> `+a;let o=document.createElement("span");o.textContent=e[0].textContent,s.append(o),l.append(s)})):l.innerHTML="<p><em>No heading structure found.</em></p>"},Ed11y.switchPanel=function(e){switch(Ed11y.panel.querySelector(".content > div:not(.hidden)")?.classList.add("hidden"),Ed11y.panel.querySelector("[aria-selected=true]")?.setAttribute("aria-selected","false"),Ed11y.panel.querySelector("#"+e).setAttribute("aria-selected","true"),Ed11y.panel.querySelector("#"+e+"-tab")?.classList.remove("hidden"),Ed11y.findElements("reset","ed11y-element-heading-label, ed11y-element-alt"),Ed11y.elements.reset?.forEach(e=>{e.remove()}),e){case"alts":Ed11y.showAltPanel();break;case"headings":Ed11y.showHeadingsPanel();break;case"help":Ed11y.showHelpPanel()}},Ed11y.alignAlts=function(){Ed11y.findElements("altMark","ed11y-element-alt"),Ed11y.elements.altMark?.forEach(e=>{var t=e.dataset.ed11yImg;e.setAttribute("style","");let i=Ed11y.imageAlts[t][0];"IMG"!==i.tagName&&(i=i.querySelector("img"));t=e.getClientRects(),t=i.getClientRects().left-t.left;let n=getComputedStyle(i).height;n="auto"===n?i.offsetHeight:Math.max(i.offsetHeight,parseInt(n)),e.setAttribute("style",`transform: translate(${t}px, 0px); height: ${n}px; width: ${i.offsetWidth}px;`)})},Ed11y.showAltPanel=function(){let o=Ed11y.panel.querySelector("#alt-list");Ed11y.imageAlts.length?(o.innerHTML="",Ed11y.imageAlts.forEach((e,t)=>{let i=document.createElement("ed11y-element-alt"),n=(i.dataset.ed11yImg=t,e[0].insertAdjacentElement("beforebegin",i),document.createElement("span")),s=(n.textContent=e[2],document.createElement("li")),a=(s.classList.add(e[3]),document.createElement("img"));a.setAttribute("src",e[1]),a.setAttribute("alt",""),s.append(a),s.append(n),o.append(s)}),Ed11y.alignAlts()):o.innerHTML="<p><em>No images found.</em></p>"},Ed11y.showHelpPanel=function(){let e=Ed11y.panel.querySelector("#help-tab");e.innerHTML=Ed11y.M.panelHelp},Ed11y.buildJumpList=function(){Ed11y.findElements("jumpList","ed11y-element-result",!1),Ed11y.elements.jumpList.forEach((e,t)=>{e.dataset.ed11yJumpPosition=t})},Ed11y.setCurrentJump=function(){var e=Ed11y.elements.jumpList.length-1;let t=0,i=(1<Ed11y.totalCount&&Ed11y.panelJumpPrev.removeAttribute("hidden"),Ed11y.goto==e||Ed11y.goto>e?(t=0,Ed11y.nextText="First"):(t=parseInt(Ed11y.goto)+1,Ed11y.nextText="Next"),t-2);i<0&&(i=1+e+i),Ed11y.panelJumpNext.dataset.ed11yGoto=t,Ed11y.panelJumpPrev.dataset.ed11yGoto=i,window.setTimeout(function(){Ed11y.panelJumpNext.querySelector(".jump-next").textContent=Ed11y.nextText},250)},Ed11y.minimize=function(){var e="true"===Ed11y.panel.classList.contains("shut");Ed11y.panel.classList.toggle("shut"),!1==e&&window.setTimeout(function(){Ed11y.panelToggle.focus()},500)},Ed11y.windowResize=function(){Ed11y.alignAlts();var e=Ed11y.getOpenTip();e&&Ed11y.alignTip(Ed11y.elements.openButton[0].shadowRoot.querySelector("button"),e)},window.addEventListener("resize",function(){Ed11y.windowResize()}),Ed11y.escapeWatch=function(e){27===e.keyCode&&(e.target.closest("ed11y-element-panel")?(Ed11y.panelToggle.focus(),Ed11y.panelToggle.click()):e.target.hasAttribute("data-ed11y-open")&&Ed11y.getOpenTip()&&(Ed11y.elements.openButton[0].shadowRoot.querySelector("button").focus(),Ed11y.elements.openButton[0].shadowRoot.querySelector("button").click()))},document.addEventListener("keyup",function(e){Ed11y.escapeWatch(e)}),Ed11y.activateTab=function(e,t){t=t||!0,Ed11y.deactivateTabs(),e.setAttribute("tabindex","0"),e.setAttribute("aria-selected","true");var i=e.getAttribute("aria-controls");Ed11y.panel.querySelector("#"+i).classList.remove("hidden"),t&&e.focus()},Ed11y.deactivateTabs=function(){Ed11y.panelTabs.forEach(e=>{e.setAttribute("tabindex","-1"),e.setAttribute("aria-selected","false")}),Ed11y.panel.querySelectorAll('[role="tabpanel"]').forEach(e=>{e.classList.add("hidden")})},Ed11y.parents=function(e){let t=[];for(t.push(e);e&&e.parentElement&&"HTML"!==e.parentElement.tagName;)t.push(e.parentElement),e=e.parentElement;return t},Ed11y.siblings=function(t){return null===t.parentNode?[]:Array.prototype.filter.call(t.parentNode.children,function(e){return e!==t})},Ed11y.nextUntil=function(e,t){let i=e.nextElementSibling;if(i){if(i.matches(t))return i;i=Ed11y.nextUntil(i,t)}return i},Ed11y.getText=function(e){return e.textContent.replace(/[\n\r]+|[\s]{2,}/g," ").trim()},Ed11y.resetClass=function(e){e?.forEach(e=>{let t=e;Ed11y.findElements("reset","."+t),Ed11y.elements.reset?.forEach(e=>{e.classList.remove(t)})})},Ed11y.keyboardClick=function(e){switch(e.preventDefault(),e.keyCode){case 13:case 32:e.target.click()}},Ed11y.visibleElement=function(t){if(t){let e=window.getComputedStyle(t);return"none"!==e.getPropertyValue("display")&&"hidden"!==e.getPropertyValue("visibility")&&"0"!==e.getPropertyValue("opacity")&&0!==t.offsetWidth&&0!==t.offsetHeight&&!t.hasAttribute("hidden")}},Ed11y.visible=function(t){if(Ed11y.visibleElement(t)){let e=Ed11y.parents(t);return e.forEach(e=>{if(!Ed11y.visibleElement(e))return!1}),!0}return!1},Ed11y.firstVisibleParent=function(e){let t=e.parentElement;return!!t&&(Ed11y.visibleElement(t)?t:t=Ed11y.firstVisibleParent(t))},Ed11y.parentLink=function(e){return e.closest("a[href]")},Ed11y.getOpenTip=function(){return Ed11y.findElements("openButton",'ed11y-element-result[data-ed11y-open="true"]'),document.querySelector('ed11y-element-tip[data-ed11y-open="true"]')},Ed11y.focusActiveResult=function(){Ed11y.getOpenTip(),window.setTimeout(function(){Ed11y.elements.openButton[0]?.shadowRoot.querySelector("button").focus()},0)},Ed11y.srcMatchesOptions=function(n,s){if(0<s.length&&0<n?.length){let e=s.split(/\s*[\s,]\s*/).map(e=>"[src*='"+e+"']"),t=e.join(", "),i=Array.from(n);return i.filter(e=>e.matches(t))}return""},Ed11y.sanitizeForHTML=function(e){let t={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;","/":"&#x2F;","`":"&#x60;","=":"&#x3D;"};return String(e).replace(/[&<>"'`=/]/g,function(e){return t[e]})},Ed11y.builder=function(e,t,i,n,s){let a=document.createElement(e);return t&&a.setAttribute("id",t),i&&("string"==typeof i?a.classList.add(i):i.forEach(e=>{a.classList.add(e)})),"object"==typeof n&&Object.entries(n).forEach(([e,t])=>{a.setAttribute(e,t)}),s&&(a.textContent=s),a},Ed11y.fnIgnore=(e,t)=>{const i=e.cloneNode(!0),n=Array.from(t?i.querySelectorAll(t):i.children);return n.forEach(e=>{e.parentElement.removeChild(e)}),i},CSS.supports("selector(:is(body))")?Ed11y.initialize():console.warn("This browser can not run Editoria11y.")}}