<?php

namespace Drupal\editoria11y;

use Drupal;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Database\Connection;

/**
 * Service description.
 */
class Api
{

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
    protected AccountInterface $account;

  /**
   * The current database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
    protected Connection $connection;

    protected EntityTypeManager $manager;

  /**
   * Constructs an Api object.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user.
   * @param \Drupal\Core\Database\Connection $connection
   *   The current database connection.
   */
    public function __construct(
        AccountInterface $account,
        Connection $connection,
        EntityTypeManager $manager
    ) {
        $this->account = $account;
        $this->connection = $connection;
        $this->manager = $manager;
    }

    public function test_results($results)
    {
        $now = time();
        //$this->validate_results($ed11y);
        // confirm result_names is array?
        foreach ($results["results"] as $key => $value) {
          // todo: handle page parameters that change content
            if ($results["page_count"] > 0) {
                $this->connection->merge("editoria11y_results")
                // Track the type and count of issues detected on this page.
                ->insertFields([
                'page_title' => $results["page_title"],
                'page_path' => $results["page_path"],
                'page_result_count' => $results["page_count"],
                'entity_type' => $results["entity_type"],
                'route_name' => $results["route_name"],
                'result_name' => $key,
                'result_name_count' => $value,
                'updated' => $now,
                'created' => $now,
                ])
                  // Update the "last seen" date of the page.
                  ->updateFields([
                  'page_title' => $results["page_title"],
                  'page_path' => $results["page_path"],
                  'page_result_count' => $results["page_count"],
                  'entity_type' => $results["entity_type"],
                  'route_name' => $results["route_name"],
                  'result_name' => $key,
                  'result_name_count' => $value,
                  'updated' => $now,
                ])
                  ->keys([
                  'page_path' => $results["page_path"],
                  'result_name' => $key,
                ])
                  ->execute();
            }


          // Update the "last seen" date for all issues that still exist.
            $this->connection->update("editoria11y_dismissals")
            ->fields([
            'stale' => 0,
            'updated' => $now,
            ])
            ->condition('page_path', $results["page_path"])
            ->condition('result_name', $key)
            ->condition('route_name', $results["route_name"])
            ->execute();
        }

      // tests marked OK are not in the result set
        foreach ($results["oks"] as $key => $value) {
          // Update the "last seen" date for all issues that still exist.
            $this->connection->update("editoria11y_dismissals")
            ->fields([
            'stale' => 0,
            'updated' => $now
            ])
            ->condition('page_path', $results["page_path"])
            ->condition('result_name', $key)
            ->condition('route_name', $results["route_name"])
            ->execute();
        }

      // Remove any test results that no longer exist
        $this->connection->delete("editoria11y_results")
        ->condition('page_path', $results["page_path"])
        ->condition('updated', $now, '!=')
        ->execute();

      // Set the stale flag for dismissals that were not updated.
      // We do not auto-delete them as some may come and go based on views.
      // todo: config or button to delete stale items to prevent creep?
        $this->connection->update("editoria11y_dismissals")
        ->fields([
        'stale' => 1
        ])
        ->condition('page_path', $results["page_path"])
        ->condition('updated', $now, '!=')
        ->execute();
        Cache::invalidateTags(['editoria11y:dashboard']);
    }

    public function dismiss(string $operation, $dismissal)
    {
        // todo: clear ignores when an OK is set?
        $now = time();
        //$this->validate_dismissals($ed11y);
        if ($operation == "reset") {
          // Reset ignores for the current user
            $this->connection->delete("editoria11y_dismissals")
            ->condition('route_name', $dismissal["route_name"])
            ->condition('page_path', $dismissal["page_path"])
            ->condition('dismissal_status', "hide")
            ->condition('uid', $this->account->id())
            ->execute();
            if ($this->account->hasPermission('Mark OK in Editoria11y Checker')) {
              // Reset "Mark OK" for the super-user
                $this->connection->delete("editoria11y_dismissals")
                ->condition('route_name', $dismissal["route_name"])
                ->condition('page_path', $dismissal["page_path"])
                ->condition('dismissal_status', "ok")
                ->execute();
            }
        } else {
            $this->connection->merge("editoria11y_dismissals")
            ->insertFields([
            'page_path' => $dismissal["page_path"],
            'page_title' => $dismissal["page_title"],
            'route_name' => $dismissal["route_name"],
            'entity_type' => $dismissal["entity_type"],
            'uid' => $this->account->id(),
            'element_id' => $dismissal["element_id"],
            'result_name' => $dismissal["result_name"],
            'result_key' => $dismissal["result_key"],
            'dismissal_status' => $operation,
            'created' => $now,
            'updated' => $now,
            ])
            ->updateFields([
            'page_path' => $dismissal["page_path"],
            'page_title' => $dismissal["page_title"],
            'route_name' => $dismissal["route_name"],
            'entity_type' => $dismissal["entity_type"],
            'uid' => $this->account->id(),
            'element_id' => $dismissal["element_id"],
            'result_name' => $dismissal["result_name"],
            'result_key' => $dismissal["result_key"],
            'dismissal_status' => $operation,
            'updated' => $now,
            ])
            ->keys([
            'element_id' => $dismissal["element_id"],
            'result_name' => $dismissal["result_name"],
            'entity_type' => $dismissal["entity_type"],
            'route_name' => $dismissal["route_name"],
            'page_path' => $dismissal["page_path"],
            ])
            ->execute();
        }
        // Clear cache for the referring page and dashboard.
        Cache::invalidateTags(['editoria11y:dismissals_' . preg_replace('/[^a-zA-Z0-9]/', '', $dismissal["page_path"]), 'editoria11y:dashboard']);
    }

    private function validate_dismissals($ed11y)
    {

      // TODO: just validate the route .

        if ($ed11y->entity_type) {
          /*$entity_type_definitions = \Drupal::entityTypeManager()->getDefinitions();

          /* @var $definition EntityTypeInterface */
          // Is the entity type valid?
         /* if (!isset($entity_type_definitions[$ed11y->entity_type])) {
            throw new Editoria11yStorageException("Content type {$ed11y->entity_type} does not exist.");
          } else if ($this->manager->getStorage($ed11y->entity_type)->load($ed11y->entity_id) === NULL) {
            throw new Editoria11yStorageException("Entity id {$ed11y->entity_id} does not exist for entity type {$ed11y->entity_type}.");
          }*/
        }
    }

    private function validate_results(\stdClass $ed11y)
    {

      // TODO: just validate the route

        if ($ed11y->entity_type) {
            $entity_type_definitions = Drupal::entityTypeManager()->getDefinitions();

          /* @var $definition EntityTypeInterface */
          // Is the entity type valid?
          /*if (!isset($entity_type_definitions[$ed11y->entity_type])) {
            throw new Editoria11yStorageException("Content type {$ed11y->entity_type} does not exist.");
          }*/

          // Is the ID a valid instance of the entity type?
          /*if ($this->manager->getStorage($ed11y->entity_type)->load($ed11y->entity_id) === NULL) {
            throw new Editoria11yStorageException("Entity id {$ed11y->entity_id} does not exist for entity type {$ed11y->entity_type}.");
          }*/
        } else {
          // todo: are we able to validate the route name?
          /*
           * entity_id: entityId, // Entity ID if exists, otherwise a path
            entity_type: drupalSettings.editoria11y.entity_type, // node or false
            route_name: drupalSettings.editoria11y.route_name, // e.g., entity.node.canonical or view.frontpage.page_1
            result_name: detail.dismissTest, // which test is sending a result
            element_id: detail.dismissKey, // some recognizable attribute of the item marked
            dismissal_status: detail.dismissAction, // ok or ignore
           * */
        }
    }
}
