<?php

namespace Drupal\editoria11y\Controller;

use Drupal\Component\Serialization\Json;

use Exception;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\Core\Controller\ControllerBase;
use Drupal\editoria11y\Api;
use Symfony\Component\HttpFoundation\Request;

class ApiController extends ControllerBase
{

  /**
   * @var Api
   */
    private $api;

  /**
   * Constructs a \Drupal\editoria11y\Api ReportsController object.
   *
   * @param $api
   *   The editoria11y API service.
   */
    public function __construct($api)
    {
        $this->api = $api;
    }

  /**
   * {@inheritdoc}
   */
    public static function create(ContainerInterface $container)
    {
        return new static(
            $container->get('editoria11y.api')
        );
    }

    public function report(Request $request)
    {
        try {
            $results = Json::decode($request->getContent());
            $this->api->test_results($results);
            return new JsonResponse("ok");
        } catch (Exception $e) {
            return $this->sendErrorResponse($results);
        }
    }

    public function ok(Request $request)
    {
        try {
            $dismissal = Json::decode($request->getContent());

            $this->api->dismiss("ok", $dismissal);
            return new JsonResponse("ok");
        } catch (Exception $e) {
            return $this->sendErrorResponse($e);
        }
    }

    public function hide(Request $request)
    {
        try {
            $dismissal = Json::decode($request->getContent());
            $this->api->dismiss("hide", $dismissal);
            return new JsonResponse("ok");
        } catch (Exception $e) {
            return $this->sendErrorResponse($e);
        }
    }

    public function reset(Request $request)
    {
        try {
            $dismissal = Json::decode($request->getContent());
            $this->api->dismiss("reset", $dismissal);
            return new JsonResponse("ok");
        } catch (Exception $e) {
            return $this->sendErrorResponse($e);
        }
    }

    private function sendErrorResponse($e)
    {
      // TODO: Record exceptions in log.
        return new JsonResponse(
            [
            "message" => "error",
            "description" => $e->getMessage(),
            "code" => $e->getCode(),
            ]
        );
    }
}
