<?php
namespace Drupal\editoria11y\Controller;

use Drupal;
use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\editoria11y\Dashboard;
use Drupal\user\Entity\User;

/**
 * Provides route responses for the Editoria11y module.
 */
class DashboardController extends ControllerBase
{
  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
    protected $account;

  /**
   * The current database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
    protected $connection;

  /**
   * @var Dashboard
   */
    protected $dashboard;

  /**
   * @param $dashboard
   */
    public function __construct($dashboard)
    {
        $this->dashboard = $dashboard;
    }

  /**
   * {@inheritdoc}
   */
    public static function create($container)
    {
        return new static(
            $container->get('editoria11y.dashboard')
        );
    }


    public function get_test_results()
    {

        $pager_manager = Drupal::service('pager.manager');
        $page = $pager_manager
        ->findPage();

        // load filters
        //$form_state = new FormState();
        //$form_state->setRebuild();

        //$render['form'] = \Drupal::formBuilder()->buildForm('Drupal\editoria11y\Form\DashboardFilters', $form_state);
        //$test = $form_state->getValue('test');

        $header = [
        ['data' => t("Issues Found"), 'field' => 't.page_result_count', 'sort' => 'desc'],
        ['data' => t("Page"), 'field' => 't.page_path'],
        ['data' => t("Type"), 'field' => 't.entity_type'],
        ];
        $rows = [];
        $results = $this->dashboard->getResults($page, $header);
        foreach ($results["results"] as $record) {
          // to review: https://drupal.stackexchange.com/questions/144992/how-do-i-create-a-link
            if (str_starts_with($record->page_path, "/")) {
                $url = Url::fromUserInput($record->page_path);
                $linkToPage = Link::fromTextAndUrl($record->page_title, $url)->toString();
            } else {
                $linkToPage = $record->page_title;
            }
            $linkString = Url::fromUserInput("/admin/reports/editoria11y/page?q=" . $record->page_path);
            $linkToIssueList = Link::fromTextAndUrl($record->page_result_count, $linkString)->toString();
            $rows[] = [$linkToIssueList, $linkToPage, $record->entity_type];
          // Todo add delete trashcan?
        }

        $render = [];
        $render[] = [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#cache' => [
          'contexts' => [
            'user',
            'url',
          ],
          'tags' => [
            'editoria11y.dashboard',
          ],
        ]
        ];
            $pager_manager
            ->createPager($results["count"], 50);
            $render[] = [
            '#type' => 'pager',
            ];


        return $render;
    }

    public function get_issues_by_page()
    {
        $header = [
          ['data' => t("Count"), 'field' => 't.result_name_count'],
          ['data' => t("Issue"), 'field' => 't.result_name', 'sort' => 'asc'],
        ];

        $rows = [];
        $key = Xss::filter(Drupal::request()->query->get('q'));
        $results = $this->dashboard->getResultsByPage($key, $header);
        $page = "";

        foreach ($results as $record) {
            $page_url = Url::fromUserInput($record->page_path);
            $page = Link::fromTextAndUrl($record->page_title, $page_url)->toString();
            $pages_by_issue_url = Url::fromUserInput("/admin/reports/editoria11y/issue?q=" . $record->result_name);
            $pages_by_issue = Link::fromTextAndUrl(t($record->result_name), $pages_by_issue_url)->toString();
            $rows[] = [$record->result_name_count, $pages_by_issue];
        }

        return [
        '#type' => 'container',
        'page_title' => [
        'element' => [
          '#markup' => "<h2>".t('Page: ') . $page ."</h2>",
        ]
        ],
        'content' => [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#cache' => [
          'contexts' => [
            'user',
            'url'
          ],
          'tags' => [
            'editoria11y.dashboard',
          ]
        ]
        ],
        ];
    }

    public function get_pages_by_issue()
    {

        $pager_manager = Drupal::service('pager.manager');
        $page = $pager_manager
        ->findPage();

        $header = [
          ['data' => t("Count"), 'field' => 'page_result_count', 'sort'=>'desc'],
          ['data' => t("Page"), 'field'=>'page_path'],
        ];

        $rows = [];
        $key = Drupal::request()->query->get('q');
        $results = $this->dashboard->getResultsByIssue($key, $page, $header);

        foreach ($results["results"] as $record) {
            if (str_starts_with($record->page_path, "/")) {
                $url = Url::fromUserInput($record->page_path);
                $link = Link::fromTextAndUrl($record->page_title, $url)->toString();
            } else {
                $link = $record->page_title;
            }
            $rows[] = [$record->result_name_count, $link];
        }

        $pager_manager
        ->createPager($results["count"], 50);

        $render = [];
        $render[] = [
        '#type' => 'container',
        'page_title' => [
        'element' => [
          '#markup' => "<h2>" . t('Pages with "' . $key . '" errors') . "</h2>",
        ]
        ],
        'content' => [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#cache' => [
          'contexts' => [
            'user',
            'url'
          ],
          'tags' => [
            'editoria11y.dashboard',
          ]
        ]
        ],
        ];
        $render[] = [
        '#type' => 'pager',
        ];
        return $render;
    }

    private function get_top_pages()
    {
        $header = [
        'Count',
        'Page',
        ];
        $rows = [];
        $results = $this->dashboard->getTopResults();
        foreach ($results as $record) {
            if (strpos($record->page_path, "/") === 0) {
                $url = Url::fromUserInput($record->page_path);
                $linkToPage = Link::fromTextAndUrl($record->page_title, $url)->toString();
            } else {
                $linkToPage = $record->page_title;
            }
            $linkString = Url::fromUserInput("/admin/reports/editoria11y/page?q=" . $record->page_path);
            $linkToIssueList = Link::fromTextAndUrl($record->page_result_count, $linkString)->toString();
            $rows[] = [$linkToIssueList, $linkToPage];
          // Todo add delete trashcan?
        }


        $render = [];

        $render[] = [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
          '#prefix' => '<h2>Pages with the most issues</h2>',
        '#cache' => [
          'contexts' => [
            'user',
            'url'
          ],
          'tags' => [
            'editoria11y.dashboard',
          ]
        ]
        ];
        $render[] = [
        'link' => [
          '#prefix' => '<p>',
          '#type' => 'link',
          '#title' => 'View all pages with issues',
          '#attributes' => ['class' => 'button'],
          '#url' => Url::fromRoute('editoria11y.reports_pages'),
          '#suffix' => '</p>',
        ],
        ];

        $render[] = [
          'spacer' => [
            '#type' => 'html_tag',
            '#tag' => 'p',
          ],
        ];

        return $render;
    }

    private function get_recent_dismissals()
    {

        $result_service = \Drupal::service('editoria11y.dashboard');
        $results = $result_service->getRecentDismissals();

//   ['page_title','route_name','page_path','created','updated','result_name','dismissal_status','uid','created','updated', 'stale']);
        $header = [
        'Page',
        'Issue',
        'Marked as',
        'User',
        'Date',
        ];

        $rows = [];

        foreach ($results as $record) {
            $user = User::load($record->uid);
            $name = $user->getDisplayName();

            if (str_starts_with($record->page_path, "/")) {
                $url = Url::fromUserInput($record->page_path);
                $link = Link::fromTextAndUrl($record->page_title, $url)->toString();
            } else {
                $link = $record->page_title;
            }
            $date = \Drupal::service('date.formatter')->format($record->created);

          // todo: return username instead of user id
          // todo: show stale as better than numerical boolean
            $rows[] = [$link, $record->result_name, $record -> dismissal_status, $name, $date];
        }

        $render = [];
        $render[] = [
        '#type' => 'container',
        'content' => [
          '#type' => 'table',
          '#prefix' => '<p>&nbsp;</p><h2>Recent dismissals</h2>',
          '#header' => $header,
          '#rows' => $rows,
          '#cache' => [
            'contexts' => [
              'user',
              'url'
            ],
            'tags' => [
              'editoria11y.dashboard',
            ]
          ],
        ],
        ];
        $render[] = [
        'link' => [
          '#prefix' => '<p>',
          '#type' => 'link',
          '#title' => 'View all dismissals',
          '#attributes' => ['class' => 'button'],
          '#url' => Url::fromRoute('editoria11y.reports_dismissals'),
          '#suffix' => '</p>',
        ],
        ];
        return $render;
    }


    private function get_top_issues($count) {
        $result_service = \Drupal::service('editoria11y.dashboard');
        $results = $result_service->getIssueList($count);
        $header = [
        'Count',
        'Issue',
        ];

        $rows = [];

        foreach ($results as $record) {
            $pages_by_issue_url = Url::fromUserInput("/admin/reports/editoria11y/issue?q=" . $record->result_name);
            $pages_by_issue = Link::fromTextAndUrl(t($record->result_name), $pages_by_issue_url)->toString();
            $rows[] = [$record->result_name_count, $pages_by_issue];
        }

        $render = [];
        $render[] = [
        '#type' => 'container',
        'content' => [
          '#type' => 'table',
          '#header' => $header,
          '#rows' => $rows,
          '#prefix' => '<p>&nbsp;</p><h2>Most frequent issues</h2>',
          '#cache' => [
            'contexts' => [
              'user',
              'url'
            ],
            'tags' => [
              'editoria11y.dashboard',
            ]
          ],
        ],
        ];
        if ($count > 0) {
          $render[] = [
            'link' => [
              '#prefix' => '<p>',
              '#type' => 'link',
              '#title' => 'View all issues',
              '#attributes' => ['class' => 'button'],
              '#url' => Url::fromRoute('editoria11y.reports_all_issues'),
              '#suffix' => '</p>',
            ],
            ];
        }
        return $render;
    }


    public function get_dismissals()
    {

      // Todo: trashcans on dashboard

        $pager_manager = Drupal::service('pager.manager');
        $page = $pager_manager
        ->findPage();

        $header = [
        'page' => t("Page with issue"),
        'issue' => t("Dismissed issue"),
        'dismissals' => t("Marked as"),
        'user' => t("By user"),
        'stale' => t('Still on page'),
        ];

        $rows = [];

        $results = $this->dashboard->getDismissals($page);

        foreach ($results["results"] as $record) {
            $user = User::load($record->uid);
            $name = $user->getDisplayName();

            if (str_starts_with($record->page_path, "/")) {
                $url = Url::fromUserInput($record->page_path);
                $link = Link::fromTextAndUrl($record->page_title, $url)->toString();
            } else {
                $link = $record->page_title;
            }

            $stale = $record -> stale ? "No" : "Yes";

          // todo: return username instead of user id
          // todo: show stale as better than numerical boolean
            $rows[] = [$link, $record->result_name, $record -> dismissal_status, $name, $stale];
        }

        $pager_manager
        ->createPager($results["count"], 50);

        $render = [];
        $render[] = [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#cache' => [
        'contexts' => [
            'user',
            'url'
          ],
          'tags' => [
            'editoria11y.dashboard',
          ]
        ]
        ];
        $render[] = [
        '#type' => 'pager',
        ];
        return $render;
    }

  /**
   * Returns a simple page.
   *
   * @return array
   *   A simple renderable array.
   */
  // todo: view by page or test, pagination
    public function dashboard(): array
    {

        return [
        '#type' => 'container',
        'content' => [
          $this->get_top_pages(),
          $this->get_top_issues(5),
          $this->get_recent_dismissals(),
        ],
        ];
    }

    /**
   * Returns a simple page.
   *
   * @return array
   *   A simple renderable array.
   */
  // todo: view by page or test, pagination
    public function results(): array
    {
        return $this->get_test_results();
    }

    /**
   * Returns a simple page.
   *
   * @return array
   *   A simple renderable array.
   */
  public function all_issues(): array
  {
      return $this->get_top_issues(0);
  }

  /**
   * Returns a simple page.
   *
   * @return array
   *   A simple renderable array.
   */
  // todo: view by page or test, pagination
    public function pages_by_issue(): array
    {
        return $this->get_pages_by_issue();
    }

  /**
   * Returns a simple page.
   *
   * @return array
   *   A simple renderable array.
   */
  // todo: view by page or test, pagination
    public function issues_by_page(): array
    {
        return $this->get_issues_by_page();
    }

  /**
   * Returns a simple page.
   *
   * @return array
   *   A simple renderable array.
   */
  // todo: view by page or test, filter by user
    public function dismissals(): array
    {
        return $this->get_dismissals();
    }
}
